@@ -1,7 +1,7 @@
 
 ##############################################################################
 #
-# Copyright (c) 2003-2018 by The University of Queensland
+# Copyright (c) 2003-2020 by The University of Queensland
 # http://www.uq.edu.au
 #
 # Primary Business: Queensland, Australia
@@ -11,12 +11,13 @@
 # Development until 2012 by Earth Systems Science Computational Center (ESSCC)
 # Development 2012-2013 by School of Earth Sciences
 # Development from 2014 by Centre for Geoscience Computing (GeoComp)
+# Development from 2019 by School of Earth and Environmental Sciences
 #
 ##############################################################################
 
 from __future__ import print_function, division
 
-__copyright__="""Copyright (c) 2003-2018 by The University of Queensland
+__copyright__="""Copyright (c) 2003-2020 by The University of Queensland
 http://www.uq.edu.au
 Primary Business: Queensland, Australia"""
 __license__="""Licensed under the Apache License, version 2.0
@@ -552,7 +553,11 @@ class Regularization(CostFunction):
         self.getPDE().setValue(X=r[1], Y=r[0])
         if not solve:
             return self.getPDE()
-        return self.getPDE().getSolution()
+        oldsettings = self.getPDE().getSolverOptions().getSolverMethod()
+        self.getPDE().getSolverOptions().setSolverMethod(SolverOptions.GMRES)
+        solution = self.getPDE().getSolution()
+        self.getPDE().getSolverOptions().setSolverMethod(oldsettings)
+        return solution
 
     def updateHessian(self):
         """
