@@ -33,15 +33,16 @@ using escript::DataTypes::real_t;
 using escript::DataTypes::cplx_t;
 
 #ifdef ESYS_HAVE_PASO
-static void addToSystemMatrixPasoCSC(paso::SystemMatrix* S,
+static void addToSystemMatrixPasoCSC(paso::SystemMatrix<double>* S,
                                      const std::vector<index_t>& Nodes,
                                      int numEq,
                                      const std::vector<double>& array);
 
-static void addToSystemMatrixPasoCSR(paso::SystemMatrix* S,
+template <typename T>
+static void addToSystemMatrixPasoCSR(paso::SystemMatrix<T>* S,
                                      const std::vector<index_t>& Nodes,
                                      int numEq,
-                                     const std::vector<double>& array);
+                                     const std::vector<T>& array);
 #endif
 
 template<>
@@ -51,7 +52,7 @@ void Assemble_addToSystemMatrix<real_t>(escript::AbstractSystemMatrix* S,
                                         const std::vector<real_t>& array)
 {
 #ifdef ESYS_HAVE_PASO
-    paso::SystemMatrix* pmat = dynamic_cast<paso::SystemMatrix*>(S);
+    paso::SystemMatrix<real_t>* pmat = dynamic_cast<paso::SystemMatrix<real_t>*>(S);
     if (pmat) {
         // call the right function depending on storage type
         if (pmat->type & MATRIX_FORMAT_CSC) {
@@ -79,6 +80,18 @@ void Assemble_addToSystemMatrix<cplx_t>(escript::AbstractSystemMatrix* S,
                                         int numEq,
                                         const std::vector<cplx_t>& array)
 {
+#ifdef ESYS_HAVE_MUMPS
+    paso::SystemMatrix<cplx_t>* pmat = dynamic_cast<paso::SystemMatrix<cplx_t>*>(S);
+    if (pmat) {
+        if (pmat->type & (MATRIX_FORMAT_OFFSET1 + MATRIX_FORMAT_BLK1)) {
+            addToSystemMatrixPasoCSR(pmat, Nodes, numEq, array);
+        } else {
+            throw DudleyException("addToSystemMatrix: MUMPS requires CSR format with "
+                                  "index offset 1 and block size 1.");
+        }
+        return;
+    }
+#endif
 #ifdef ESYS_HAVE_TRILINOS
     TrilinosMatrixAdapter* tmat = dynamic_cast<TrilinosMatrixAdapter*>(S);
     if (tmat) {
@@ -86,12 +99,12 @@ void Assemble_addToSystemMatrix<cplx_t>(escript::AbstractSystemMatrix* S,
         return;
     }
 #endif
-    throw DudleyException("addToSystemMatrix: only Trilinos matrices support "
+    throw DudleyException("addToSystemMatrix: require Trilinos or MUMPS matrices for "
                           "complex-valued assembly!");
 }
 
 #ifdef ESYS_HAVE_PASO
-void addToSystemMatrixPasoCSC(paso::SystemMatrix* in,
+void addToSystemMatrixPasoCSC(paso::SystemMatrix<double>* in,
                               const std::vector<index_t>& Nodes,
                               int numEq, const std::vector<double>& array)
 {
@@ -191,9 +204,10 @@ void addToSystemMatrixPasoCSC(paso::SystemMatrix* in,
     }
 }
 
-void addToSystemMatrixPasoCSR(paso::SystemMatrix* in,
+template <typename T>
+void addToSystemMatrixPasoCSR(paso::SystemMatrix<T>* in,
                               const std::vector<index_t>& Nodes,
-                              int numEq, const std::vector<double>& array)
+                              int numEq, const std::vector<T>& array)
 {
     const int index_offset = (in->type & MATRIX_FORMAT_OFFSET1 ? 1 : 0);
     const int row_block_size = in->row_block_size;
@@ -207,13 +221,13 @@ void addToSystemMatrixPasoCSR(paso::SystemMatrix* in,
 
     const index_t* mainBlock_ptr = in->mainBlock->pattern->ptr;
     const index_t* mainBlock_index = in->mainBlock->pattern->index;
-    double* mainBlock_val = in->mainBlock->val;
+    T* mainBlock_val = in->mainBlock->val;
     const index_t* col_coupleBlock_ptr = in->col_coupleBlock->pattern->ptr;
     const index_t* col_coupleBlock_index = in->col_coupleBlock->pattern->index;
-    double* col_coupleBlock_val = in->col_coupleBlock->val;
+    T* col_coupleBlock_val = in->col_coupleBlock->val;
     const index_t* row_coupleBlock_ptr = in->row_coupleBlock->pattern->ptr;
     const index_t* row_coupleBlock_index = in->row_coupleBlock->pattern->index;
-    double* row_coupleBlock_val = in->row_coupleBlock->val;
+    T* row_coupleBlock_val = in->row_coupleBlock->val;
 
     for (int k_Eq = 0; k_Eq < NN; ++k_Eq) { // Down columns of array
         const index_t j_Eq = Nodes[k_Eq];
