@@ -1491,10 +1491,17 @@ escript::ASM_ptr DudleyDomain::newSystemMatrix(int row_blocksize,
     } else if (type & (int)SMT_PASO) {
 #ifdef ESYS_HAVE_PASO
         paso::SystemMatrixPattern_ptr pattern(getPasoPattern());
-        paso::SystemMatrix_ptr sm(new paso::SystemMatrix(type, pattern,
-                  row_blocksize, column_blocksize, false, row_functionspace,
-                  column_functionspace));
-        return sm;
+        if (type & (int)MATRIX_FORMAT_COMPLEX) {
+            paso::SystemMatrix_ptr<cplx_t> sm(new paso::SystemMatrix<cplx_t>(type, pattern,
+                      row_blocksize, column_blocksize, false, row_functionspace,
+                      column_functionspace));
+            return sm;
+        } else {
+            paso::SystemMatrix_ptr<double> sm(new paso::SystemMatrix<double>(type, pattern,
+                      row_blocksize, column_blocksize, false, row_functionspace,
+                      column_functionspace));
+            return sm;
+        }
 #else
         throw DudleyException("newSystemMatrix: dudley was not compiled "
                 "with Paso support so the Paso solver stack cannot be used.");
@@ -1780,11 +1787,18 @@ int DudleyDomain::getSystemMatrixTypeId(const bp::object& options) const
     }
 #ifdef ESYS_HAVE_PASO
     if (sb.isComplex()) {
-        throw NotImplementedError("Paso does not support complex-valued matrices");
+#ifdef ESYS_HAVE_MUMPS
+        return (int)SMT_PASO | paso::SystemMatrix<cplx_t>::getSystemMatrixTypeId(
+                    method, sb.getPreconditioner(), sb.getPackage(),
+                    sb.isComplex(), sb.isSymmetric(), m_mpiInfo);
+#else
+        throw NotImplementedError("Paso requires MUMPS for complex-valued matrices.");
+#endif
+    } else {
+        return (int)SMT_PASO | paso::SystemMatrix<double>::getSystemMatrixTypeId(
+                    method, sb.getPreconditioner(), sb.getPackage(),
+                    sb.isComplex(), sb.isSymmetric(), m_mpiInfo);
     }
-    return (int)SMT_PASO | paso::SystemMatrix::getSystemMatrixTypeId(
-                method, sb.getPreconditioner(), sb.getPackage(),
-                sb.isSymmetric(), m_mpiInfo);
 #else
     throw DudleyException("Unable to find a working solver library!");
 #endif
