@@ -870,8 +870,9 @@ class Test_FaultSystemOnDudley(unittest.TestCase):
       f.addFault(V0=[10.,0,0],strikes=3*pi/4,ls=14.142135623730951, dips=pi/4, depths=50.,tag=2)
       self.assertTrue( [ 1, 2 ] == f.getTags(), "tags wrong")
       self.assertTrue(  abs(14.142135623730951 - f.getTotalLength(2)) <self.EPS, "length wrong")
-      self.assertTrue(  50. == f.getMediumDepth(2), "depth wrong")
-      self.assertTrue( (-50., 0.) ==  f.getW1Range(2)," wrong W1 range")
+      self.assertTrue( abs(50. - f.getMediumDepth(2)) <= self.EPS, "depth wrong")
+      self.assertTrue( abs(-50. - f.getW1Range(2)[0]) <= self.EPS," wrong W1 range (0)")
+      self.assertTrue( abs(0. - f.getW1Range(2)[1]) <= self.EPS," wrong W1 range (1)")
       segs=f.getTopPolyline(2)
       self.assertTrue( len(segs) == 2, "wrong number of segments")
       self.assertTrue( isinstance(segs[0], numpy.ndarray), "wrong class of vertex 0")
