@@ -1,7 +1,7 @@
 
 ##############################################################################
 #
-# Copyright (c) 2003-2018 by The University of Queensland
+# Copyright (c) 2003-2020 by The University of Queensland
 # http://www.uq.edu.au
 #
 # Primary Business: Queensland, Australia
@@ -11,12 +11,13 @@
 # Development until 2012 by Earth Systems Science Computational Center (ESSCC)
 # Development 2012-2013 by School of Earth Sciences
 # Development from 2014 by Centre for Geoscience Computing (GeoComp)
+# Development from 2019 by School of Earth and Environmental Sciences
 #
 ##############################################################################
 
 from __future__ import print_function, division
 
-__copyright__="""Copyright (c) 2003-2018 by The University of Queensland
+__copyright__="""Copyright (c) 2003-2020 by The University of Queensland
 http://www.uq.edu.au
 Primary Business: Queensland, Australia"""
 __license__="""Licensed under the Apache License, version 2.0
@@ -484,6 +485,7 @@ class SolverSchemeException(Exception):
    """
    pass
 
+
 class IndefinitePreconditioner(SolverSchemeException):
    """
    Exception thrown if the preconditioner is not positive definite.
@@ -573,7 +575,7 @@ def PCG(r, Aprod, x, Msolve, bilinearform, atol=0, rtol=1.e-8, iter_max=100, ini
    norm_r0=math.sqrt(rhat_dot_r)
    atol2=atol+rtol*norm_r0
    if atol2<=0:
-      raise ValueError("Non-positive tolarance.")
+      raise ValueError("Non-positive tolerance.")
    atol2=max(atol2, 100. * util.EPSILON * norm_r0)
 
    if verbose: print(("PCG: initial residual norm = %e (absolute tolerance = %e)"%(norm_r0, atol2)))
