@@ -169,7 +169,7 @@ namespace DataTypes
                       Region in other vector to copy data from.
   */
    template <class VEC>
-   ESCRIPT_DLL_API
+   // ESCRIPT_DLL_API
    void
    copySlice(VEC& left,
              const ShapeType& leftShape,
@@ -276,7 +276,7 @@ namespace DataTypes
                       Region in the left vector to copy data to.
   */
    template<typename VEC>
-   ESCRIPT_DLL_API
+   // ESCRIPT_DLL_API
    void
    copySliceFrom(VEC& left,
                  const ShapeType& leftShape,
