@@ -26,7 +26,7 @@ namespace bp = boost::python;
 
 namespace escript {
 
-EscriptParams escriptParams; // externed in header file
+ESCRIPT_DLL_API EscriptParams escriptParams; // externed in header file
 
 
 EscriptParams::EscriptParams()
@@ -96,6 +96,9 @@ EscriptParams::EscriptParams()
 #ifdef ESYS_HAVE_UMFPACK
     features.insert("umfpack");
 #endif
+#ifdef ESYS_HAVE_MUMPS
+    features.insert("mumps");
+#endif
 #ifdef ESYS_HAVE_WEIPA
     features.insert("weipa");
 #endif
@@ -117,6 +120,10 @@ EscriptParams::EscriptParams()
 #ifdef ESYS_HAVE_BOOST_NUMPY
     features.insert("boostnumpy");
 #endif
+
+#ifndef ESYS_NO_SYMPY
+    features.insert("sympy");
+#endif
 }
 
 int EscriptParams::getInt(const std::string& name, int sentinel) const
@@ -178,7 +185,7 @@ bool EscriptParams::hasFeature(const std::string& name) const
         if (MPI_Comm_size(MPI_COMM_WORLD, &size) != MPI_SUCCESS || size > 1)
             return false;
 #endif
-        return hasFeature("paso") && (hasFeature("umfpack") || hasFeature("mkl"));
+        return hasFeature("paso") && (hasFeature("umfpack") || hasFeature("mkl") || hasFeature("mumps"));
     }
 
     return features.count(name) > 0;
