@@ -18,6 +18,8 @@
 #ifndef __ESCRIPT_ESYSMPI_H__
 #define __ESCRIPT_ESYSMPI_H__
 
+#include "system_dep.h"
+
 #include <escript/DataTypes.h>
 
 #include <ctime>
@@ -75,9 +77,10 @@ typedef boost::shared_ptr<JMPI_> JMPI;
 
 /// creates a JMPI shared pointer from MPI communicator
 /// if owncom is true, the communicator is freed when mpi info is destroyed.
+ESCRIPT_DLL_API
 JMPI makeInfo(MPI_Comm comm, bool owncom=false);
 
-class JMPI_
+class ESCRIPT_DLL_API JMPI_
 {
 public:
     ~JMPI_();
@@ -158,7 +161,7 @@ public:
 
 private:
     JMPI_(MPI_Comm comm, bool owncomm);
-    friend JMPI makeInfo(MPI_Comm comm, bool owncom);
+    friend ESCRIPT_DLL_API JMPI makeInfo(MPI_Comm comm, bool owncom);
 #pragma clang diagnostic push
 #pragma clang diagnostic ignored "-Wunused-private-field"
     bool ownscomm;
@@ -176,11 +179,13 @@ public:
 };
 
 /// Everyone puts in their error code and everyone gets the largest one
+ESCRIPT_DLL_API
 bool checkResult(int input, int& output, const JMPI& comm);
 
 /// ensure that the any ranks with an empty src argument end up with the
 /// string from one of the other ranks.
 /// With no MPI, it makes dest point at a copy of src.
+ESCRIPT_DLL_API
 bool shipString(const char* src, char** dest, MPI_Comm& comm);
 
 /// returns the current ticks for timing
