@@ -17,14 +17,18 @@
 #ifndef __ESCRIPT_RANDOM_H__
 #define __ESCRIPT_RANDOM_H__
 
+#include "system_dep.h"
+
 namespace escript
 {
 /* \brief put n random doubles (from [0.0, 1.0]) in array (uses OpenMP).
    If using this on Data, then be sure to CHECK_EX_WRITE first
 */
+ESCRIPT_DLL_API
 void randomFillArray(long seed, double* array, size_t n);
 
 
+ESCRIPT_DLL_API
 void patternFillArray2D(size_t x, size_t y, double* array, size_t spacing,
                         size_t basex, size_t basey, size_t numpoints);
 
