@@ -20,6 +20,10 @@
 
 #include <boost/python.hpp>
 
+#ifdef ESYS_HAVE_TRILINOS
+#include <Amesos2.hpp>
+#endif
+
 namespace bp = boost::python;
 
 template <class R>
@@ -127,6 +131,7 @@ const char* SolverBuddy::getName(int key) const
         case SO_PACKAGE_PASO: return "PASO";
         case SO_PACKAGE_TRILINOS: return "TRILINOS";
         case SO_PACKAGE_UMFPACK: return "UMFPACK";
+        case SO_PACKAGE_MUMPS: return "MUMPS";
 
         case SO_METHOD_BICGSTAB: return "BICGSTAB";
         case SO_METHOD_CGLS: return "CGLS";
@@ -333,8 +338,8 @@ void SolverBuddy::setPreconditioner(int precon)
     SolverOptions preconditioner = static_cast<SolverOptions>(precon);
     switch(preconditioner) {
         case SO_PRECONDITIONER_AMG:
-#ifndef ESYS_HAVE_TRILINOS
-        throw ValueError("escript was not compiled with Trilinos enabled");
+#if !defined(ESYS_HAVE_TRILINOS) && !defined(ESYS_HAVE_MUMPS)
+        throw ValueError("escript was not compiled with Trilinos or MUMPS enabled");
 #endif
         case SO_PRECONDITIONER_GAUSS_SEIDEL:
         case SO_PRECONDITIONER_JACOBI: // This is the default preconditioner in ifpack2
@@ -361,7 +366,7 @@ void SolverBuddy::setSolverMethod(int method)
 
 //     bool havePASODirect = false;
 //     using_default_solver_method=false;
-// #if defined(ESYS_HAVE_PASO) && (defined(ESYS_HAVE_MKL) || defined(ESYS_HAVE_UMFPACK))
+// #if defined(ESYS_HAVE_PASO) && (defined(ESYS_HAVE_MKL) || defined(ESYS_HAVE_UMFPACK) || defined(ESYS_HAVE_MUMPS))
 //     havePASODirect = true;
 // #endif
 
@@ -385,21 +390,48 @@ void SolverBuddy::setSolverMethod(int method)
             this->method = meth;
             break;
         case SO_METHOD_DIRECT:
-        case SO_METHOD_DIRECT_MUMPS:
-        case SO_METHOD_DIRECT_PARDISO:
-        case SO_METHOD_DIRECT_SUPERLU:
-        case SO_METHOD_DIRECT_TRILINOS:
-#if defined(ESYS_HAVE_UMFPACK) || defined(ESYS_HAVE_TRILINOS) || defined(ESYS_HAVE_MKL)
-#ifndef ESYS_HAVE_TRILINOS
+#if defined(ESYS_HAVE_UMFPACK) || defined(ESYS_HAVE_TRILINOS) || defined(ESYS_HAVE_MKL) || defined(ESYS_HAVE_MUMPS)
+#ifdef ESYS_HAVE_TRILINOS
             // translate specific direct solver setting to generic one for PASO
-            this->method = SO_METHOD_DIRECT;
-#else
             this->method = meth;
+#else
+            this->method = SO_METHOD_DIRECT;
 #endif
             break;
 #else
             throw ValueError("Cannot use DIRECT solver method, the running "
                     "escript was not compiled with a direct solver enabled");
+#endif
+        case SO_METHOD_DIRECT_TRILINOS:
+#ifdef ESYS_HAVE_TRILINOS
+            this->method = meth;
+            break;
+#else
+            throw ValueError("escript was not compiled with Trilinos");
+#endif
+        case SO_METHOD_DIRECT_MUMPS:
+#ifdef ESYS_HAVE_MUMPS
+            this->method=meth;
+#else
+            throw ValueError("escript was not compiled with MUMPS");
+#endif
+        case SO_METHOD_DIRECT_PARDISO:
+#ifdef ESYS_HAVE_TRILINOS
+            if(Amesos2::query("pardiso_mkl"))
+                this->method = meth;
+            else
+                throw ValueError("Trilinos was not compiled with MKL Pardiso");
+#else
+            throw ValueError("escript was not compiled with Trilinos");
+#endif
+        case SO_METHOD_DIRECT_SUPERLU:
+#ifdef ESYS_HAVE_TRILINOS
+            if(Amesos2::query("superludist") || Amesos2::query("superlu") || Amesos2::query("superlumt"))
+                this->method = meth;
+            else
+                throw ValueError("Trilinos was not compiled with SuperLU ");
+#else
+            throw ValueError("escript was not compiled with Trilinos");
 #endif
         default:
             throw ValueError("unknown solver method");
@@ -463,6 +495,15 @@ void SolverBuddy::setPackage(int package)
             break;
 #else
             throw ValueError("escript was not compiled with UMFPACK enabled");
+#endif
+        // Set to MUMPS iff escript was compiled with MUMPS
+        case SO_PACKAGE_MUMPS:
+#ifdef ESYS_HAVE_MUMPS
+            this->package = SO_PACKAGE_MUMPS;
+            setSolverMethod(getSolverMethod());
+            break;
+#else
+            throw ValueError("escript was not compiled with MUMPS enabled");
 #endif
         default:
             throw ValueError("unknown solver package");
