@@ -35,6 +35,7 @@ SO_PACKAGE_MKL: Intel's MKL solver library
 SO_PACKAGE_PASO: PASO solver package
 SO_PACKAGE_TRILINOS: The TRILINOS parallel solver class library from Sandia National Labs
 SO_PACKAGE_UMFPACK: The UMFPACK library
+SO_PACKAGE_MUMPS: The MUMPS library
 
 SO_METHOD_BICGSTAB: The stabilized Bi-Conjugate Gradient method
 SO_METHOD_CHOLEVSKY: The direct solver based on LDLT factorization (can only be applied for symmetric PDEs)
@@ -90,6 +91,7 @@ enum SolverOptions
     SO_PACKAGE_PASO,
     SO_PACKAGE_TRILINOS,
     SO_PACKAGE_UMFPACK,
+    SO_PACKAGE_MUMPS,
 
     // Solver methods
     SO_METHOD_BICGSTAB,
@@ -289,7 +291,7 @@ public:
         \param package key of the solver package to be used, should be in
                `SO_DEFAULT`, `SO_PACKAGE_CUSP`, `SO_PACKAGE_PASO`,
                `SO_PACKAGE_MKL`, `SO_PACKAGE_UMFPACK`,
-               `SO_PACKAGE_TRILINOS`
+               `SO_PACKAGE_TRILINOS`, `SO_PACKAGE_MUMPS`
 
         \note Not all packages are supported on all implementation.
               An exception may be thrown on some platforms if the selected
