@@ -22,7 +22,9 @@
 
 #include <cstring>
 #include <fstream>
-#include <unistd.h>
+#ifndef _WIN32
+# include <unistd.h>
+#endif // _WIN32
 
 #include <boost/python.hpp>
 #include <boost/scoped_array.hpp>
@@ -569,53 +571,61 @@ void saveDataCSV(const std::string& filename, bp::dict arg,
     try {
         std::vector<int> offset(numdata);
         std::vector<const DataTypes::real_t*> samples(numdata);
-
-	const DataTypes::real_t onlyreal=0;
-        for (int i=0; i<numsamples; ++i) {
-            if (!best.ownSample(i)) {
+#ifdef ESYS_MPI
+        // Loop over the MPI ranks
+        for(int token = 0; token < data[0].getDomain()->getMPISize(); token++)
+        {
+            data[0].getDomain()->MPIBarrier();
+            if(data[0].getDomain()->getMPIRank() != token)
                 continue;
-            }
-            wantrow = true;
-            for (int d=0; d<numdata; ++d) {
-                samples[d] = data[d].getSampleDataRO(i, onlyreal);
-            }
-            if (hasmask) {
-                masksample = mask.getSampleDataRO(i, onlyreal);
-                if (!expandedmask) {
-                    // mask controls whole sample
-                    if (masksample[0] <= 0) {
-                        wantrow = false;
+#endif
+
+	    	const DataTypes::real_t onlyreal=0;
+            for (int i=0; i<numsamples; ++i) {
+                if (!best.ownSample(i)) {
+                    continue;
+                }
+                wantrow = true;
+                for (int d=0; d<numdata; ++d) {
+                    samples[d] = data[d].getSampleDataRO(i, onlyreal);
+                }
+                if (hasmask) {
+                    masksample = mask.getSampleDataRO(i, onlyreal);
+                    if (!expandedmask) {
+                        // mask controls whole sample
+                        if (masksample[0] <= 0) {
+                            wantrow = false;
+                        }
                     }
                 }
-            }
-            for (int j=0; j<dpps; ++j) {
-                // now we need to check if this point is masked off
-                if (expandedmask) {
-                    // masks are scalar so the relevant value is at [j]
-                    wantrow = (masksample[j]>0);
+                for (int j=0; j<dpps; ++j) {
+                    // now we need to check if this point is masked off
+                    if (expandedmask) {
+                        // masks are scalar so the relevant value is at [j]
+                        wantrow = (masksample[j]>0);
+                    }
+                    if (wantrow) {
+                        bool needsep = false;
+                        // If necessary, write the element id to the output stream
+                        if(refid){
+                            os << best.getReferenceIDOfSample(i) << sep;
+                        }
+                        for (int d = 0; d < numdata; ++d) {
+                            DataTypes::pointToStream(os, samples[d],
+                                data[d].getDataPointShape(), offset[d], needsep, sep);
+                            needsep = true;
+                            offset[d] += step[d];
+                        }
+                        os << std::endl;
+                    }
                 }
-                if (wantrow) {
-                    bool needsep = false;
-
-                    // If necessary, write the element id to the output stream
-                    if(refid){
-                        os << best.getReferenceIDOfSample(i) << sep;
-                    }
-
-                    for (int d = 0; d < numdata; ++d) {
-                        DataTypes::pointToStream(os, samples[d],
-                            data[d].getDataPointShape(), offset[d], needsep, sep);
-                        needsep = true;
-                        offset[d] += step[d];
-                    }
-                    os << std::endl;
+                for (int d = 0; d<numdata; d++) {
+                    offset[d]=0;
                 }
             }
-
-            for (int d = 0; d<numdata; d++) {
-                offset[d]=0;
-            }
+#ifdef ESYS_MPI
         }
+#endif
     } catch (EsysException* e) {
         error=1;
         if (data[0].getDomain()->getMPISize()==1) {
