@@ -28,7 +28,9 @@
 #include "FunctionSpaceFactory.h"
 #include "MPIDataReducer.h"
 #include "MPIScalarReducer.h"
-#include "NCHelper.h"
+#ifdef NETCDF4
+  #include "NCHelper.h"
+#endif
 #include "NonReducedVariable.h"
 #include "SolverOptions.h"
 #include "SplitWorld.h"
@@ -120,14 +122,19 @@ BOOST_PYTHON_MODULE(escriptcpp)
 // params are: bool show_user_defined, bool show_py_signatures, bool show_cpp_signatures
     docstring_options docopt(true,true,false);
 #endif
+#ifdef ESYS_HAVE_BOOST_NUMPY
+    numpy::initialize();
+#endif
 
     scope().attr("__doc__") = "To use this module, please import esys.escript";
 
     // register escript's default translators
     REGISTER_ESCRIPT_EXCEPTION_TRANSLATORS;
-
-
+    
+    
+#ifdef NETCDF4
     def("NcFType", escript::NcFType, arg("filename"), "Return a character indicating what netcdf format a file uses.\nc or C indicates netCDF3.\n4 indicates netCDF4.\nu indicates unsupported format (eg netCDF4 file in an escript build which does not support it\n? indicates unknown.");
+#endif
 
 /* begin SubWorld things */
 
@@ -139,8 +146,9 @@ BOOST_PYTHON_MODULE(escriptcpp)
   def("internal_addJob", raw_function(escript::raw_addJob,2));
   def("internal_addJobPerWorld", raw_function(escript::raw_addJobPerWorld,2));
   def("internal_addVariable", raw_function(escript::raw_addVariable,3));
-
-
+// #ifdef ESYS_HAVE_BOOST_NUMPY
+//   def("internal_initBoostNumpy", &escript::initBoostNumpy);
+// #endif
   def("internal_makeDataReducer", escript::makeDataReducer, arg("op"), "Create a reducer to work with Data and the specified operation.");
   def("internal_makeScalarReducer", escript::makeScalarReducer, arg("op"), "Create a reducer to work with doubles and the specified operation.");
   def("internal_makeLocalOnly", escript::makeNonReducedVariable, "Create a variable which is not connected to copies in other worlds.");
@@ -215,6 +223,15 @@ BOOST_PYTHON_MODULE(escriptcpp)
         ":param arg: Data object\n"
         ":rtype: numpy ndarray\n"
         "");
+// #ifdef ESYS_HAVE_BOOST_NUMPY
+//   def("_numpyToData", escript::numpyToData,(arg("array"), arg("isComplex"), arg("functionspace")),
+//         "Takes in a numpy ndarray and function space and returns a Data object\n"
+//         ":param array: A numpy ndarray\n"
+//         ":param isComplex: boolean. True for complex data \n"
+//         ":param functionspace: A FunctionSpace\n"
+//         ":rtype: Data object\n"
+//         "");
+// #endif
   def("canInterpolate", &escript::canInterpolate, args("src", "dest"),":param src: Source FunctionSpace\n"
         ":param dest: Destination FunctionSpace\n"
         ":return: True if src can be interpolated to dest\n"
@@ -1061,6 +1078,7 @@ args("source", "q", "r","factor"),
     .value("PASO", escript::SO_PACKAGE_PASO)
     .value("TRILINOS", escript::SO_PACKAGE_TRILINOS)
     .value("UMFPACK", escript::SO_PACKAGE_UMFPACK)
+    .value("MUMPS", escript::SO_PACKAGE_MUMPS)
 
     .value("BICGSTAB", escript::SO_METHOD_BICGSTAB)
     .value("CGLS", escript::SO_METHOD_CGLS)
@@ -1173,10 +1191,10 @@ args("source", "q", "r","factor"),
         ":rtype: in the list `DEFAULT`, `DIRECT`, `CHOLEVSKY`, `PCG`, `CR`, `CGS`, `BICGSTAB`, `GMRES`, `PRES20`, `ROWSUM_LUMPING`, `HRZ_LUMPING`, `MINRES`, `ITERATIVE`, `NONLINEAR_GMRES`, `TFQMR`")
     .def("setPackage", &escript::SolverBuddy::setPackage, args("package"),"Sets the solver package to be used as a solver.\n\n"
         ":param package: key of the solver package to be used.\n"
-        ":type package: in `DEFAULT`, `PASO`, `CUSP`, `MKL`, `UMFPACK`, `TRILINOS`\n"
+        ":type package: in `DEFAULT`, `PASO`, `CUSP`, `MKL`, `UMFPACK`, `MUMPS`, `TRILINOS`\n"
         ":note: Not all packages are support on all implementation. An exception may be thrown on some platforms if a particular package is requested.")
     .def("getPackage", &escript::SolverBuddy::getPackage,"Returns the solver package key\n\n"
-        ":rtype: in the list `DEFAULT`, `PASO`, `CUSP`, `MKL`, `UMFPACK`, `TRILINOS`")
+        ":rtype: in the list `DEFAULT`, `PASO`, `CUSP`, `MKL`, `UMFPACK`, `MUMPS`, `TRILINOS`")
     .def("setReordering", &escript::SolverBuddy::setReordering, args("ordering"),"Sets the key of the reordering method to be applied if supported by the solver. Some direct solvers support reordering to optimize compute time and storage use during elimination.\n\n"
         ":param ordering: selects the reordering strategy.\n"
         ":type ordering: in 'NO_REORDERING', 'MINIMUM_FILL_IN', 'NESTED_DISSECTION', 'DEFAULT_REORDERING'")
