@@ -1267,9 +1267,6 @@ cerr << "Ending sin" << endl;
       }
     }
   }
-#if defined (_WIN32) && !defined (__INTEL_COMPILER)
-  cout << "\tSkip test Data::erf on windows with MSVC compiler." << endl;
-#else
   cout << "\tTest Data::erf." << endl;
   for (int z=0;z<NUMDATS;++z)
   {
@@ -1293,7 +1290,6 @@ cerr << "Ending sin" << endl;
       }
     }
   }
-#endif
 
 
   cout << "\tTest Data::abs." << endl;
