@@ -33,15 +33,16 @@ using escript::DataTypes::real_t;
 using escript::DataTypes::cplx_t;
 
 #ifdef ESYS_HAVE_PASO
-static void addToSystemMatrixPasoCSC(paso::SystemMatrix* S, int NN_Equa,
+static void addToSystemMatrixPasoCSC(paso::SystemMatrix<double>* S, int NN_Equa,
                                      const index_t* Nodes_Equa, int num_Equa,
                                      int NN_Sol, const index_t* Nodes_Sol,
                                      int num_Sol, const real_t* array);
 
-static void addToSystemMatrixPasoCSR(paso::SystemMatrix* S, int NN_Equa,
+template <typename T>
+static void addToSystemMatrixPasoCSR(paso::SystemMatrix<T>* S, int NN_Equa,
                                     const index_t* Nodes_Equa, int num_Equa,
                                     int NN_Sol, const index_t* Nodes_Sol,
-                                    int num_Sol, const real_t* array);
+                                    int num_Sol, const T* array);
 #endif
 
 template<>
@@ -51,7 +52,7 @@ void Assemble_addToSystemMatrix<real_t>(escript::ASM_ptr S, int NN_Equa,
                                     int num_Sol, const real_t* array)
 {
 #ifdef ESYS_HAVE_PASO
-    paso::SystemMatrix* pmat = dynamic_cast<paso::SystemMatrix*>(S.get());
+    paso::SystemMatrix<real_t>* pmat = dynamic_cast<paso::SystemMatrix<real_t>*>(S.get());
     if (pmat) {
         // call the right function depending on storage type
         if (pmat->type & MATRIX_FORMAT_CSC) {
@@ -86,6 +87,20 @@ void Assemble_addToSystemMatrix<cplx_t>(escript::ASM_ptr S, int NN_Equa,
                                     int NN_Sol, const index_t* Nodes_Sol,
                                     int num_Sol, const cplx_t* array)
 {
+#ifdef ESYS_HAVE_MUMPS
+    paso::SystemMatrix<cplx_t>* pmat = dynamic_cast<paso::SystemMatrix<cplx_t>*>(S.get());
+    if (pmat) {
+        if (pmat->type & (MATRIX_FORMAT_OFFSET1 + MATRIX_FORMAT_BLK1)) {
+            addToSystemMatrixPasoCSR(pmat, NN_Equa, Nodes_Equa,
+                                     num_Equa, NN_Sol, Nodes_Sol,
+                                     num_Sol, array);
+        } else {
+            throw FinleyException("addToSystemMatrix: MUMPS requires CSR format with "
+                                  "index offset 1 and block size 1.");
+        }
+        return;
+    }
+#endif
 #ifdef ESYS_HAVE_TRILINOS
     TrilinosMatrixAdapter* tmat = dynamic_cast<TrilinosMatrixAdapter*>(S.get());
     if (tmat) {
@@ -101,7 +116,7 @@ void Assemble_addToSystemMatrix<cplx_t>(escript::ASM_ptr S, int NN_Equa,
 }
 
 #ifdef ESYS_HAVE_PASO
-void addToSystemMatrixPasoCSC(paso::SystemMatrix* in, int NN_Equa,
+void addToSystemMatrixPasoCSC(paso::SystemMatrix<double>* in, int NN_Equa,
                               const index_t* Nodes_Equa, int num_Equa,
                               int NN_Sol, const index_t* Nodes_Sol,
                               int num_Sol, const real_t* array)
@@ -203,10 +218,11 @@ void addToSystemMatrixPasoCSC(paso::SystemMatrix* in, int NN_Equa,
     }
 }
 
-void addToSystemMatrixPasoCSR(paso::SystemMatrix* in, int NN_Equa,
+template <typename T>
+void addToSystemMatrixPasoCSR(paso::SystemMatrix<T>* in, int NN_Equa,
                               const index_t* Nodes_Equa, int num_Equa,
                               int NN_Sol, const index_t* Nodes_Sol,
-                              int num_Sol, const real_t* array)
+                              int num_Sol, const T* array)
 {
     const int index_offset = (in->type & MATRIX_FORMAT_OFFSET1 ? 1 : 0);
     const int row_block_size = in->row_block_size;
@@ -219,13 +235,13 @@ void addToSystemMatrixPasoCSR(paso::SystemMatrix* in, int NN_Equa,
 
     const index_t* mainBlock_ptr = in->mainBlock->pattern->ptr;
     const index_t* mainBlock_index = in->mainBlock->pattern->index;
-    real_t* mainBlock_val = in->mainBlock->val;
+    T* mainBlock_val = in->mainBlock->val;
     const index_t* col_coupleBlock_ptr = in->col_coupleBlock->pattern->ptr;
     const index_t* col_coupleBlock_index = in->col_coupleBlock->pattern->index;
-    real_t* col_coupleBlock_val = in->col_coupleBlock->val;
+    T* col_coupleBlock_val = in->col_coupleBlock->val;
     const index_t* row_coupleBlock_ptr = in->row_coupleBlock->pattern->ptr;
     const index_t* row_coupleBlock_index = in->row_coupleBlock->pattern->index;
-    real_t* row_coupleBlock_val = in->row_coupleBlock->val;
+    T* row_coupleBlock_val = in->row_coupleBlock->val;
 
     for (int k_Equa = 0; k_Equa < NN_Equa; ++k_Equa) {
         // Down columns of array
