@@ -27,7 +27,7 @@ __url__="https://launchpad.net/escript-finley"
 Test suite for PDE solvers on finley
 """
 
-from test_simplesolve import SimpleSolveTestCase
+from test_simplesolve import SimpleSolveTestCase, SimpleSolveTestCaseOrder2
 import esys.escriptcore.utestselect as unittest
 from esys.escriptcore.testing import *
 
@@ -47,86 +47,9 @@ OPTIMIZE=True
 class SimpleSolveOnPaso(SimpleSolveTestCase):
     pass
 
-class SimpleSolveOrder2(SimpleSolveOnPaso):
-    def _getGrad(self, system):
-        """returns exact gradient"""
-        dim = self.domain.getDim()
-        x = Solution(self.domain).getX()
-        if system:
-            g_ex = Data(0., (dim,dim), Solution(self.domain))
-            if dim == 2:
-                g_ex[0,0] = 2.+8.*x[0]+ 5.*x[1]
-                g_ex[0,1] = 3.+5.*x[0]+12.*x[1]
-                g_ex[1,0] = 4.+2.*x[0]+ 6.*x[1]
-                g_ex[1,1] = 2.+6.*x[0]+ 8.*x[1]
-            else:
-                g_ex[0,0] =  2.+6.*x[1]+8.*x[2]+18.*x[0]
-                g_ex[0,1] =  3.+6.*x[0]+7.*x[2]+20.*x[1]
-                g_ex[0,2] =  4.+7.*x[1]+8.*x[0]+22.*x[2]
-                g_ex[1,0] =  4.+3.*x[1]-8.*x[2]- 4.*x[0]
-                g_ex[1,1] =  1.+3.*x[0]+2.*x[2]+14.*x[1]
-                g_ex[1,2] = -6.+2.*x[1]-8.*x[0]+10.*x[2]
-                g_ex[2,0] =  7.-6.*x[1]+2.*x[2]+ 4.*x[0]
-                g_ex[2,1] =  9.-6.*x[0]+8.*x[2]+16.*x[1]
-                g_ex[2,2] =  2.+8.*x[1]+2.*x[0]+ 2.*x[2]
-        else:
-            g_ex = Data(0., (dim,), Solution(self.domain))
-            if dim == 2:
-                g_ex[0] = 2.+8.*x[0]+5.*x[1]
-                g_ex[1] = 3.+5.*x[0]+12.*x[1]
-            else:
-                g_ex[0] = 2.+6.*x[1]+8.*x[2]+18.*x[0]
-                g_ex[1] = 3.+6.*x[0]+7.*x[2]+20.*x[1]
-                g_ex[2] = 4.+7.*x[1]+8.*x[0]+22.*x[2]
-        return g_ex
-
-    def _getSolution(self, system):
-        """returns exact solution"""
-        dim = self.domain.getDim()
-        x = Solution(self.domain).getX()
-        if system:
-            u_ex = Vector(0., Solution(self.domain))
-            if dim == 2:
-                u_ex[0] =  1.+2.*x[0]+3.*x[1]+4.*x[0]**2+5.*x[1]*x[0]+6.*x[1]**2
-                u_ex[1] = -1.+4.*x[0]+2.*x[1]+1.*x[0]**2+6.*x[1]*x[0]+4.*x[1]**2
-            else:
-                u_ex[0] = 1.+2.*x[0]+3.*x[1]+4.*x[2]+\
-                          6.*x[0]*x[1]+7.*x[1]*x[2]+8.*x[2]*x[0]+\
-                          9.*x[0]**2+10.*x[1]**2+11.*x[2]**2
-                u_ex[1] = 2.+4.*x[0]+1.*x[1]-6.*x[2]+\
-                          3.*x[0]*x[1]+2.*x[1]*x[2]-8.*x[2]*x[0]-\
-                          2.*x[0]**2+7.*x[1]**2+5.*x[2]**2
-                u_ex[2] = -2.+7.*x[0]+9.*x[1]+2*x[2]-\
-                          6.*x[0]*x[1]+8.*x[1]*x[2]+2.*x[2]*x[0]+\
-                          2.*x[0]**2+8.*x[1]**2+1.*x[2]**2
-        else:
-            if dim == 2:
-                u_ex = 1.+2.*x[0]+3.*x[1]+4.*x[0]**2+5.*x[1]*x[0]+6.*x[1]**2
-            else:
-                u_ex = 1.+2.*x[0]+3.*x[1]+4.*x[2]+\
-                       6.*x[0]*x[1]+7.*x[1]*x[2]+8.*x[2]*x[0]+\
-                       9.*x[0]**2+10.*x[1]**2+11.*x[2]**2
-        return u_ex
-
-    def _setCoefficients(self, pde, system):
-        """sets PDE coefficients"""
-        super(SimpleSolveOrder2, self)._setCoefficients(pde, system)
-        dim = self.domain.getDim()
-        if system:
-            Y = pde.getCoefficient("Y")
-            if dim == 2:
-                Y[0] = Y[0]-20.
-                Y[1] = Y[1]-10.
-            else:
-                Y[0] = Y[0]-60.
-                Y[1] = Y[1]-20.
-                Y[2] = Y[2]-22.
-            pde.setValue(Y=Y)
-        else:
-            if dim == 2:
-                pde.setValue(Y=-20.)
-            else:
-                pde.setValue(Y=-60.)
+@unittest.skipIf(not HAVE_PASO, "PASO not available")
+class SimpleSolveOnPasoOrder2(SimpleSolveTestCaseOrder2):
+    pass
 
 ### BiCGStab + Jacobi
 
@@ -140,7 +63,7 @@ class Test_SimpleSolveFinleyRect_Order1_Paso_BICGSTAB_Jacobi(SimpleSolveOnPaso):
     def tearDown(self):
         del self.domain
 
-class Test_SimpleSolveFinleyRect_Order2_Paso_BICGSTAB_Jacobi(SimpleSolveOrder2):
+class Test_SimpleSolveFinleyRect_Order2_Paso_BICGSTAB_Jacobi(SimpleSolveOnPasoOrder2):
     def setUp(self):
         self.domain = Rectangle(NE0, NE1, 2, optimize=OPTIMIZE)
         self.package = SolverOptions.PASO
@@ -160,7 +83,7 @@ class Test_SimpleSolveFinleyBrick_Order1_Paso_BICGSTAB_Jacobi(SimpleSolveOnPaso)
     def tearDown(self):
         del self.domain
 
-class Test_SimpleSolveFinleyBrick_Order2_Paso_BICGSTAB_Jacobi(SimpleSolveOrder2):
+class Test_SimpleSolveFinleyBrick_Order2_Paso_BICGSTAB_Jacobi(SimpleSolveOnPasoOrder2):
     def setUp(self):
         self.domain = Brick(NE0, NE1, NE2, 2, optimize=OPTIMIZE)
         self.package = SolverOptions.PASO
@@ -182,7 +105,7 @@ class Test_SimpleSolveFinleyRect_Order1_Paso_PCG_Jacobi(SimpleSolveOnPaso):
     def tearDown(self):
         del self.domain
 
-class Test_SimpleSolveFinleyRect_Order2_Paso_PCG_Jacobi(SimpleSolveOrder2):
+class Test_SimpleSolveFinleyRect_Order2_Paso_PCG_Jacobi(SimpleSolveOnPasoOrder2):
     def setUp(self):
         self.domain = Rectangle(NE0, NE1, 2, optimize=OPTIMIZE)
         self.package = SolverOptions.PASO
@@ -202,7 +125,7 @@ class Test_SimpleSolveFinleyBrick_Order1_Paso_PCG_Jacobi(SimpleSolveOnPaso):
     def tearDown(self):
         del self.domain
 
-class Test_SimpleSolveFinleyBrick_Order2_Paso_PCG_Jacobi(SimpleSolveOrder2):
+class Test_SimpleSolveFinleyBrick_Order2_Paso_PCG_Jacobi(SimpleSolveOnPasoOrder2):
     def setUp(self):
         self.domain = Brick(NE0, NE1, NE2, 2, optimize=OPTIMIZE)
         self.package = SolverOptions.PASO
@@ -224,7 +147,7 @@ class Test_SimpleSolveFinleyRect_Order1_Paso_TFQMR_Jacobi(SimpleSolveOnPaso):
     def tearDown(self):
         del self.domain
 
-class Test_SimpleSolveFinleyRect_Order2_Paso_TFQMR_Jacobi(SimpleSolveOrder2):
+class Test_SimpleSolveFinleyRect_Order2_Paso_TFQMR_Jacobi(SimpleSolveOnPasoOrder2):
     def setUp(self):
         self.domain = Rectangle(NE0, NE1, 2, optimize=OPTIMIZE)
         self.package = SolverOptions.PASO
@@ -244,7 +167,7 @@ class Test_SimpleSolveFinleyBrick_Order1_Paso_TFQMR_Jacobi(SimpleSolveOnPaso):
     def tearDown(self):
         del self.domain
 
-class Test_SimpleSolveFinleyBrick_Order2_Paso_TFQMR_Jacobi(SimpleSolveOrder2):
+class Test_SimpleSolveFinleyBrick_Order2_Paso_TFQMR_Jacobi(SimpleSolveOnPasoOrder2):
     def setUp(self):
         self.domain = Brick(NE0, NE1, NE2, 2, optimize=OPTIMIZE)
         self.package = SolverOptions.PASO
@@ -266,7 +189,7 @@ class Test_SimpleSolveFinleyRect_Order1_Paso_MINRES_Jacobi(SimpleSolveOnPaso):
     def tearDown(self):
         del self.domain
 
-class Test_SimpleSolveFinleyRect_Order2_Paso_MINRES_Jacobi(SimpleSolveOrder2):
+class Test_SimpleSolveFinleyRect_Order2_Paso_MINRES_Jacobi(SimpleSolveOnPasoOrder2):
     def setUp(self):
         self.domain = Rectangle(NE0, NE1, 2, optimize=OPTIMIZE)
         self.package = SolverOptions.PASO
@@ -286,7 +209,7 @@ class Test_SimpleSolveFinleyBrick_Order1_Paso_MINRES_Jacobi(SimpleSolveOnPaso):
     def tearDown(self):
         del self.domain
 
-class Test_SimpleSolveFinleyBrick_Order2_Paso_MINRES_Jacobi(SimpleSolveOrder2):
+class Test_SimpleSolveFinleyBrick_Order2_Paso_MINRES_Jacobi(SimpleSolveOnPasoOrder2):
     def setUp(self):
         self.domain = Brick(NE0, NE1, NE2, 2, optimize=OPTIMIZE)
         self.package = SolverOptions.PASO
@@ -308,7 +231,7 @@ class Test_SimpleSolveFinleyRect_Order1_Paso_BICGSTAB_GaussSeidel(SimpleSolveOnP
     def tearDown(self):
         del self.domain
 
-class Test_SimpleSolveFinleyRect_Order2_Paso_BICGSTAB_GaussSeidel(SimpleSolveOrder2):
+class Test_SimpleSolveFinleyRect_Order2_Paso_BICGSTAB_GaussSeidel(SimpleSolveOnPasoOrder2):
     def setUp(self):
         self.domain = Rectangle(NE0, NE1, 2, optimize=OPTIMIZE)
         self.package = SolverOptions.PASO
@@ -328,7 +251,7 @@ class Test_SimpleSolveFinleyBrick_Order1_Paso_BICGSTAB_GaussSeidel(SimpleSolveOn
     def tearDown(self):
         del self.domain
 
-class Test_SimpleSolveFinleyBrick_Order2_Paso_BICGSTAB_GaussSeidel(SimpleSolveOrder2):
+class Test_SimpleSolveFinleyBrick_Order2_Paso_BICGSTAB_GaussSeidel(SimpleSolveOnPasoOrder2):
     def setUp(self):
         self.domain = Brick(NE0, NE1, NE2, 2, optimize=OPTIMIZE)
         self.package = SolverOptions.PASO
@@ -350,7 +273,7 @@ class Test_SimpleSolveFinleyRect_Order1_Paso_PCG_GaussSeidel(SimpleSolveOnPaso):
     def tearDown(self):
         del self.domain
 
-class Test_SimpleSolveFinleyRect_Order2_Paso_PCG_GaussSeidel(SimpleSolveOrder2):
+class Test_SimpleSolveFinleyRect_Order2_Paso_PCG_GaussSeidel(SimpleSolveOnPasoOrder2):
     def setUp(self):
         self.domain = Rectangle(NE0, NE1, 2, optimize=OPTIMIZE)
         self.package = SolverOptions.PASO
@@ -370,7 +293,7 @@ class Test_SimpleSolveFinleyBrick_Order1_Paso_PCG_GaussSeidel(SimpleSolveOnPaso)
     def tearDown(self):
         del self.domain
 
-class Test_SimpleSolveFinleyBrick_Order2_Paso_PCG_GaussSeidel(SimpleSolveOrder2):
+class Test_SimpleSolveFinleyBrick_Order2_Paso_PCG_GaussSeidel(SimpleSolveOnPasoOrder2):
     def setUp(self):
         self.domain = Brick(NE0, NE1, NE2, 2, optimize=OPTIMIZE)
         self.package = SolverOptions.PASO
@@ -392,7 +315,7 @@ class Test_SimpleSolveFinleyRect_Order1_Paso_TFQMR_GaussSeidel(SimpleSolveOnPaso
     def tearDown(self):
         del self.domain
 
-class Test_SimpleSolveFinleyRect_Order2_Paso_TFQMR_GaussSeidel(SimpleSolveOrder2):
+class Test_SimpleSolveFinleyRect_Order2_Paso_TFQMR_GaussSeidel(SimpleSolveOnPasoOrder2):
     def setUp(self):
         self.domain = Rectangle(NE0, NE1, 2, optimize=OPTIMIZE)
         self.package = SolverOptions.PASO
@@ -412,7 +335,7 @@ class Test_SimpleSolveFinleyBrick_Order1_Paso_TFQMR_GaussSeidel(SimpleSolveOnPas
     def tearDown(self):
         del self.domain
 
-class Test_SimpleSolveFinleyBrick_Order2_Paso_TFQMR_GaussSeidel(SimpleSolveOrder2):
+class Test_SimpleSolveFinleyBrick_Order2_Paso_TFQMR_GaussSeidel(SimpleSolveOnPasoOrder2):
     def setUp(self):
         self.domain = Brick(NE0, NE1, NE2, 2, optimize=OPTIMIZE)
         self.package = SolverOptions.PASO
@@ -434,7 +357,7 @@ class Test_SimpleSolveFinleyRect_Order1_Paso_MINRES_GaussSeidel(SimpleSolveOnPas
     def tearDown(self):
         del self.domain
 
-class Test_SimpleSolveFinleyRect_Order2_Paso_MINRES_GaussSeidel(SimpleSolveOrder2):
+class Test_SimpleSolveFinleyRect_Order2_Paso_MINRES_GaussSeidel(SimpleSolveOnPasoOrder2):
     def setUp(self):
         self.domain = Rectangle(NE0, NE1, 2, optimize=OPTIMIZE)
         self.package = SolverOptions.PASO
@@ -454,7 +377,7 @@ class Test_SimpleSolveFinleyBrick_Order1_Paso_MINRES_GaussSeidel(SimpleSolveOnPa
     def tearDown(self):
         del self.domain
 
-class Test_SimpleSolveFinleyBrick_Order2_Paso_MINRES_GaussSeidel(SimpleSolveOrder2):
+class Test_SimpleSolveFinleyBrick_Order2_Paso_MINRES_GaussSeidel(SimpleSolveOnPasoOrder2):
     def setUp(self):
         self.domain = Brick(NE0, NE1, NE2, 2, optimize=OPTIMIZE)
         self.package = SolverOptions.PASO
@@ -476,7 +399,7 @@ class Test_SimpleSolveFinleyRect_Order1_Paso_BICGSTAB_RILU(SimpleSolveOnPaso):
     def tearDown(self):
         del self.domain
 
-class Test_SimpleSolveFinleyRect_Order2_Paso_BICGSTAB_RILU(SimpleSolveOrder2):
+class Test_SimpleSolveFinleyRect_Order2_Paso_BICGSTAB_RILU(SimpleSolveOnPasoOrder2):
     def setUp(self):
         self.domain = Rectangle(NE0, NE1, 2, optimize=OPTIMIZE)
         self.package = SolverOptions.PASO
@@ -496,7 +419,7 @@ class Test_SimpleSolveFinleyBrick_Order1_Paso_BICGSTAB_RILU(SimpleSolveOnPaso):
     def tearDown(self):
         del self.domain
 
-class Test_SimpleSolveFinleyBrick_Order2_Paso_BICGSTAB_RILU(SimpleSolveOrder2):
+class Test_SimpleSolveFinleyBrick_Order2_Paso_BICGSTAB_RILU(SimpleSolveOnPasoOrder2):
     def setUp(self):
         self.domain = Brick(NE0, NE1, NE2, 2, optimize=OPTIMIZE)
         self.package = SolverOptions.PASO
@@ -518,7 +441,7 @@ class Test_SimpleSolveFinleyRect_Order1_Paso_PCG_RILU(SimpleSolveOnPaso):
     def tearDown(self):
         del self.domain
 
-class Test_SimpleSolveFinleyRect_Order2_Paso_PCG_RILU(SimpleSolveOrder2):
+class Test_SimpleSolveFinleyRect_Order2_Paso_PCG_RILU(SimpleSolveOnPasoOrder2):
     def setUp(self):
         self.domain = Rectangle(NE0, NE1, 2, optimize=OPTIMIZE)
         self.package = SolverOptions.PASO
@@ -538,7 +461,7 @@ class Test_SimpleSolveFinleyBrick_Order1_Paso_PCG_RILU(SimpleSolveOnPaso):
     def tearDown(self):
         del self.domain
 
-class Test_SimpleSolveFinleyBrick_Order2_Paso_PCG_RILU(SimpleSolveOrder2):
+class Test_SimpleSolveFinleyBrick_Order2_Paso_PCG_RILU(SimpleSolveOnPasoOrder2):
     def setUp(self):
         self.domain = Brick(NE0, NE1, NE2, 2, optimize=OPTIMIZE)
         self.package = SolverOptions.PASO
@@ -560,7 +483,7 @@ class Test_SimpleSolveFinleyRect_Order1_Paso_TFQMR_RILU(SimpleSolveOnPaso):
     def tearDown(self):
         del self.domain
 
-class Test_SimpleSolveFinleyRect_Order2_Paso_TFQMR_RILU(SimpleSolveOrder2):
+class Test_SimpleSolveFinleyRect_Order2_Paso_TFQMR_RILU(SimpleSolveOnPasoOrder2):
     def setUp(self):
         self.domain = Rectangle(NE0, NE1, 2, optimize=OPTIMIZE)
         self.package = SolverOptions.PASO
@@ -580,7 +503,7 @@ class Test_SimpleSolveFinleyBrick_Order1_Paso_TFQMR_RILU(SimpleSolveOnPaso):
     def tearDown(self):
         del self.domain
 
-class Test_SimpleSolveFinleyBrick_Order2_Paso_TFQMR_RILU(SimpleSolveOrder2):
+class Test_SimpleSolveFinleyBrick_Order2_Paso_TFQMR_RILU(SimpleSolveOnPasoOrder2):
     def setUp(self):
         self.domain = Brick(NE0, NE1, NE2, 2, optimize=OPTIMIZE)
         self.package = SolverOptions.PASO
@@ -602,7 +525,7 @@ class Test_SimpleSolveFinleyRect_Order1_Paso_MINRES_RILU(SimpleSolveOnPaso):
     def tearDown(self):
         del self.domain
 
-class Test_SimpleSolveFinleyRect_Order2_Paso_MINRES_RILU(SimpleSolveOrder2):
+class Test_SimpleSolveFinleyRect_Order2_Paso_MINRES_RILU(SimpleSolveOnPasoOrder2):
     def setUp(self):
         self.domain = Rectangle(NE0, NE1, 2, optimize=OPTIMIZE)
         self.package = SolverOptions.PASO
@@ -622,7 +545,7 @@ class Test_SimpleSolveFinleyBrick_Order1_Paso_MINRES_RILU(SimpleSolveOnPaso):
     def tearDown(self):
         del self.domain
 
-class Test_SimpleSolveFinleyBrick_Order2_Paso_MINRES_RILU(SimpleSolveOrder2):
+class Test_SimpleSolveFinleyBrick_Order2_Paso_MINRES_RILU(SimpleSolveOnPasoOrder2):
     def setUp(self):
         self.domain = Brick(NE0, NE1, NE2, 2, optimize=OPTIMIZE)
         self.package = SolverOptions.PASO
