@@ -1,7 +1,7 @@
 
 /*****************************************************************************
 *
-* Copyright (c) 2003-2020 by The University of Queensland
+* Copyright (c) 2003-2018 by The University of Queensland
 * http://www.uq.edu.au
 *
 * Primary Business: Queensland, Australia
@@ -10,9 +10,8 @@
 *
 * Development until 2012 by Earth Systems Science Computational Center (ESSCC)
 * Development 2012-2013 by School of Earth Sciences
-* Development from 2014-2017 by Centre for Geoscience Computing (GeoComp)
-* Development from 2019 by School of Earth and Environmental Sciences
-**
+* Development from 2014 by Centre for Geoscience Computing (GeoComp)
+*
 *****************************************************************************/
 
 
@@ -147,7 +146,7 @@ SolverResult FCT_Solver::updateLCN(double* u, double* u_old, Options* options,
     dim_t sweep_max, i;
     double const RTOL = options->tolerance;
     const dim_t n = transportproblem->getTotalNumRows();
-    SystemMatrix_ptr iteration_matrix(transportproblem->iteration_matrix);
+    SystemMatrix_ptr<double> iteration_matrix(transportproblem->iteration_matrix);
     const index_t* main_iptr = transportproblem->borrowMainDiagonalPointer();
     SolverResult errorCode = NoError;
     double norm_u_tilde;
@@ -383,7 +382,7 @@ SolverResult FCT_Solver::updateNL(double* u, double* u_old, Options* options,
  *   for BE : theta = 1.
  */
 
-void FCT_Solver::setAntiDiffusionFlux_CN(SystemMatrix_ptr flux_matrix)
+void FCT_Solver::setAntiDiffusionFlux_CN(SystemMatrix_ptr<double> flux_matrix)
 {
     const double* u = u_coupler->borrowLocalData();
     const double* u_old = u_old_coupler->borrowLocalData();
@@ -434,7 +433,7 @@ void FCT_Solver::setAntiDiffusionFlux_CN(SystemMatrix_ptr flux_matrix)
     }
 }
 
-void FCT_Solver::setAntiDiffusionFlux_BE(SystemMatrix_ptr flux_matrix)
+void FCT_Solver::setAntiDiffusionFlux_BE(SystemMatrix_ptr<double> flux_matrix)
 {
     const double* u = u_coupler->borrowLocalData();
     const double* u_old = u_old_coupler->borrowLocalData();
@@ -490,7 +489,7 @@ void FCT_Solver::setAntiDiffusionFlux_BE(SystemMatrix_ptr flux_matrix)
  *    =  2*  m_{ij} * ( (u_old[j]-u_tilde[j] - (u_old[i]) - u_tilde[i]) ) - dt d_{ij} * (u_tilde[j]-u_tilde[i])
  *
  */
-void FCT_Solver::setAntiDiffusionFlux_linearCN(SystemMatrix_ptr flux_matrix)
+void FCT_Solver::setAntiDiffusionFlux_linearCN(SystemMatrix_ptr<double> flux_matrix)
 {
     const_Coupler_ptr<real_t> u_tilde_coupler(flux_limiter->u_tilde_coupler);
     const double* u_tilde = u_tilde_coupler->borrowLocalData();
@@ -600,7 +599,7 @@ void FCT_Solver::setLowOrderOperator(TransportProblem_ptr fc)
     const index_t* main_iptr = fc->borrowMainDiagonalPointer();
 
     if (!fc->iteration_matrix.get()) {
-        fc->iteration_matrix.reset(new SystemMatrix(
+        fc->iteration_matrix.reset(new SystemMatrix<double>(
                   fc->transport_matrix->type, fc->transport_matrix->pattern,
                   fc->transport_matrix->row_block_size,
                   fc->transport_matrix->col_block_size, true,
@@ -666,7 +665,7 @@ void FCT_Solver::setLowOrderOperator(TransportProblem_ptr fc)
  */
 void FCT_Solver::setMuPaLu(double* out, const_Coupler_ptr<real_t> coupler, double a)
 {
-    const_SystemMatrix_ptr L(transportproblem->iteration_matrix);
+    const_SystemMatrix_ptr<double> L(transportproblem->iteration_matrix);
     const double* M = transportproblem->lumped_mass_matrix;
     const_SystemMatrixPattern_ptr pattern(L->pattern);
     const double* u = coupler->borrowLocalData();
