@@ -24,8 +24,7 @@
 
 namespace paso {
 
-PASO_DLL_API
-struct FCT_Solver
+struct PASO_DLL_API FCT_Solver
 {
     FCT_Solver(const_TransportProblem_ptr tp, Options* options);
 
@@ -43,11 +42,11 @@ struct FCT_Solver
 
     static void setLowOrderOperator(TransportProblem_ptr tp);
 
-    void setAntiDiffusionFlux_linearCN(SystemMatrix_ptr flux_matrix);
+    void setAntiDiffusionFlux_linearCN(SystemMatrix_ptr<double> flux_matrix);
 
-    void setAntiDiffusionFlux_BE(SystemMatrix_ptr flux_matrix);
+    void setAntiDiffusionFlux_BE(SystemMatrix_ptr<double> flux_matrix);
 
-    void setAntiDiffusionFlux_CN(SystemMatrix_ptr flux_matrix);
+    void setAntiDiffusionFlux_CN(SystemMatrix_ptr<double> flux_matrix);
 
     void setMuPaLu(double* out, const_Coupler_ptr<real_t> coupler, double a);
 
