@@ -1,7 +1,7 @@
 
 /*****************************************************************************
 *
-* Copyright (c) 2003-2020 by The University of Queensland
+* Copyright (c) 2003-2018 by The University of Queensland
 * http://www.uq.edu.au
 *
 * Primary Business: Queensland, Australia
@@ -10,9 +10,8 @@
 *
 * Development until 2012 by Earth Systems Science Computational Center (ESSCC)
 * Development 2012-2013 by School of Earth Sciences
-* Development from 2014-2017 by Centre for Geoscience Computing (GeoComp)
-* Development from 2019 by School of Earth and Environmental Sciences
-**
+* Development from 2014 by Centre for Geoscience Computing (GeoComp)
+*
 *****************************************************************************/
 
 
@@ -42,7 +41,7 @@ FCT_FluxLimiter::FCT_FluxLimiter(const_TransportProblem_ptr tp)
 
     R_coupler.reset(new Coupler<real_t>(tp->borrowConnector(), 2*blockSize, mpi_info));
     u_tilde_coupler.reset(new Coupler<real_t>(tp->borrowConnector(), blockSize, mpi_info));
-    antidiffusive_fluxes.reset(new SystemMatrix(
+    antidiffusive_fluxes.reset(new SystemMatrix<double>(
                 tp->transport_matrix->type, tp->transport_matrix->pattern,
                 tp->transport_matrix->row_block_size,
                 tp->transport_matrix->col_block_size, true,
@@ -140,7 +139,7 @@ void FCT_FluxLimiter::addLimitedFluxes_Start()
     const dim_t n = getTotalNumRows();
     const_SystemMatrixPattern_ptr pattern(getFluxPattern());
     const double* remote_u_tilde = u_tilde_coupler->borrowRemoteData();
-    SystemMatrix_ptr adf(antidiffusive_fluxes);
+    SystemMatrix_ptr<double> adf(antidiffusive_fluxes);
 
 #pragma omp parallel for
     for (dim_t i = 0; i < n; ++i) {
@@ -209,7 +208,7 @@ void FCT_FluxLimiter::addLimitedFluxes_Complete(double* b)
 {
     const dim_t n = getTotalNumRows();
     const_SystemMatrixPattern_ptr pattern(getFluxPattern());
-    const_SystemMatrix_ptr adf(antidiffusive_fluxes);
+    const_SystemMatrix_ptr<double> adf(antidiffusive_fluxes);
     const double* remote_R = R_coupler->finishCollect();
 
 #pragma omp parallel for
