@@ -23,8 +23,7 @@
 namespace paso {
 
 
-PASO_DLL_API
-struct FCT_FluxLimiter
+struct PASO_DLL_API FCT_FluxLimiter
 {
     FCT_FluxLimiter(const_TransportProblem_ptr tp);
     ~FCT_FluxLimiter();
@@ -43,7 +42,7 @@ struct FCT_FluxLimiter
     void addLimitedFluxes_Start();
     void addLimitedFluxes_Complete(double* b);
 
-    SystemMatrix_ptr antidiffusive_fluxes;
+    SystemMatrix_ptr<double> antidiffusive_fluxes;
     escript::JMPI mpi_info;
     double dt;
     double* u_tilde;
