@@ -1,7 +1,7 @@
 
 /*****************************************************************************
 *
-* Copyright (c) 2003-2020 by The University of Queensland
+* Copyright (c) 2003-2018 by The University of Queensland
 * http://www.uq.edu.au
 *
 * Primary Business: Queensland, Australia
@@ -10,9 +10,8 @@
 *
 * Development until 2012 by Earth Systems Science Computational Center (ESSCC)
 * Development 2012-2013 by School of Earth Sciences
-* Development from 2014-2017 by Centre for Geoscience Computing (GeoComp)
-* Development from 2019 by School of Earth and Environmental Sciences
-**
+* Development from 2014 by Centre for Geoscience Computing (GeoComp)
+*
 *****************************************************************************/
 
 
@@ -42,7 +41,7 @@ void Solver_ILU_free(Solver_ILU * in)
 }
 
 /// constructs the incomplete block factorization
-Solver_ILU* Solver_getILU(SparseMatrix_ptr A, bool verbose)
+Solver_ILU* Solver_getILU(SparseMatrix_ptr<double> A, bool verbose)
 {
     const dim_t n=A->numRows;
     const dim_t n_block=A->row_block_size;
@@ -314,7 +313,7 @@ Solver_ILU* Solver_getILU(SparseMatrix_ptr A, bool verbose)
    vector is available.
 */
 
-void Solver_solveILU(SparseMatrix_ptr A, Solver_ILU* ilu, double* x,
+void Solver_solveILU(SparseMatrix_ptr<double> A, Solver_ILU* ilu, double* x,
                      const double* b)
 {
     dim_t i,k;
