@@ -1,7 +1,7 @@
 
 /*****************************************************************************
 *
-* Copyright (c) 2003-2020 by The University of Queensland
+* Copyright (c) 2003-2018 by The University of Queensland
 * http://www.uq.edu.au
 *
 * Primary Business: Queensland, Australia
@@ -10,9 +10,8 @@
 *
 * Development until 2012 by Earth Systems Science Computational Center (ESSCC)
 * Development 2012-2013 by School of Earth Sciences
-* Development from 2014-2017 by Centre for Geoscience Computing (GeoComp)
-* Development from 2019 by School of Earth and Environmental Sciences
-**
+* Development from 2014 by Centre for Geoscience Computing (GeoComp)
+*
 *****************************************************************************/
 
 
@@ -57,7 +56,7 @@ namespace paso {
 *  ==============================================================
 */
 
-SolverResult Solver_MINRES(SystemMatrix_ptr A, double* R, double* X,
+SolverResult Solver_MINRES(SystemMatrix_ptr<double> A, double* R, double* X,
                            dim_t* iter, double* tolerance, Performance* pp)
 {
     const dim_t maxit = *iter;
