@@ -1,7 +1,7 @@
 
 /*****************************************************************************
 *
-* Copyright (c) 2003-2020 by The University of Queensland
+* Copyright (c) 2003-2018 by The University of Queensland
 * http://www.uq.edu.au
 *
 * Primary Business: Queensland, Australia
@@ -10,9 +10,8 @@
 *
 * Development until 2012 by Earth Systems Science Computational Center (ESSCC)
 * Development 2012-2013 by School of Earth Sciences
-* Development from 2014-2017 by Centre for Geoscience Computing (GeoComp)
-* Development from 2019 by School of Earth and Environmental Sciences
-**
+* Development from 2014 by Centre for Geoscience Computing (GeoComp)
+*
 *****************************************************************************/
 
 
@@ -33,7 +32,7 @@
 
 namespace paso {
 
-void MKL_free(SparseMatrix* A)
+void MKL_free(SparseMatrix<double>* A)
 {
 #ifdef ESYS_HAVE_MKL
     if (A && A->solver_p && A->solver_package==PASO_MKL) {
@@ -64,7 +63,7 @@ void MKL_free(SparseMatrix* A)
 #endif
 }
 
-void MKL_solve(SparseMatrix_ptr A, double* out, double* in, index_t reordering,
+void MKL_solve(SparseMatrix_ptr<double> A, double* out, double* in, index_t reordering,
                dim_t numRefinements, bool verbose)
 {
 #ifdef ESYS_HAVE_MKL
@@ -173,5 +172,24 @@ void MKL_solve(SparseMatrix_ptr A, double* out, double* in, index_t reordering,
 #endif
 }
 
+void MKL_free(SparseMatrix<cplx_t>* A)
+{
+#ifdef ESYS_HAVE_MKL
+    throw PasoException("Paso MKL_free(): complex not implemented.");
+#else
+    throw PasoException("Paso: MKL is not available.");
+#endif
+}
+
+void MKL_solve(SparseMatrix_ptr<cplx_t> A, cplx_t* out, cplx_t* in, index_t reordering,
+               dim_t numRefinements, bool verbose)
+{
+#ifdef ESYS_HAVE_MKL
+    throw PasoException("Paso MKL_solve(): complex not implemented.");
+#else
+    throw PasoException("Paso: MKL is not available.");
+#endif
+}
+
 } // namespace paso
 
