@@ -54,6 +54,7 @@
 #define PASO_NESTED_DISSECTION 19
 #define PASO_ITERATIVE 20
 #define PASO_PASO 21
+#define PASO_MUMPS 22
 #define PASO_REC_ILU  23
 #define PASO_TRILINOS  24
 #define PASO_NONLINEAR_GMRES  25
@@ -75,7 +76,7 @@
 
 namespace paso {
 
-struct Options
+struct PASO_DLL_API Options
 {
     Options() { setDefaults(); }
 
