@@ -30,6 +30,7 @@
 #ifndef __PASO_H__
 #define __PASO_H__
 
+#include "system_dep.h"
 #include <escript/index.h>
 #include <escript/DataTypes.h>
 #include <escript/EsysMPI.h>
@@ -57,13 +58,12 @@ using escript::DataTypes::cplx_t;
 
 }
 
-#define PASO_DLL_API
-
 #define MATRIX_FORMAT_DEFAULT 1
 #define MATRIX_FORMAT_CSC 2
 #define MATRIX_FORMAT_BLK1 4
 #define MATRIX_FORMAT_OFFSET1 8
 #define MATRIX_FORMAT_DIAGONAL_BLOCK 32
+#define MATRIX_FORMAT_COMPLEX 64
 
 #define PASO_ONE (double)(1.0)
 #define PASO_ZERO (double)(0.0)
