@@ -30,6 +30,7 @@
 #define __PASO_PATTERN_H__
 
 #include "Paso.h"
+#include "PasoException.h"
 
 #include <escript/IndexList.h>
 
@@ -39,7 +40,7 @@ struct Pattern;
 typedef boost::shared_ptr<Pattern> Pattern_ptr;
 typedef boost::shared_ptr<const Pattern> const_Pattern_ptr;
 
-struct Pattern : boost::enable_shared_from_this<Pattern>
+struct PASO_DLL_API Pattern : boost::enable_shared_from_this<Pattern>
 {
     Pattern(int type, dim_t numOutput, dim_t numInput, index_t* ptr,
             index_t* index);
@@ -102,6 +103,30 @@ struct Pattern : boost::enable_shared_from_this<Pattern>
         return deg;
     }
 
+    // convert csr row ptr and col indices to harwell-boeing format
+    inline void csrToHB()
+    {
+        // TODO: add openmp
+        if (! (type & (MATRIX_FORMAT_OFFSET1 + MATRIX_FORMAT_BLK1)) ) {
+            throw PasoException(
+                "Paso: Harwell-Boeing format requires CSR format with index offset 1 and block size 1.");
+        }
+
+        if ( !(hb_row == NULL && hb_col == NULL) ) {
+            return;
+        }
+
+        hb_row = new index_t[len];
+        hb_col = new index_t[len];
+        for (dim_t i=0, k=0; i<numOutput; i++)
+        {
+            for (dim_t j=ptr[i]; j<ptr[i+1]; j++, k++)
+            {
+                hb_row[k] = i+1;
+                hb_col[k] = index[j-1];
+            }
+        }
+    }
 
     int type;
     // Number of rows in the ptr array [CSR] / number of cols for CSC
@@ -120,6 +145,10 @@ struct Pattern : boost::enable_shared_from_this<Pattern>
     dim_t numColors;
     // coloring index: inputs with the same color are not connected
     index_t* coloring;
+    // row indices in harwell-boeing format
+    index_t* hb_row;
+    // col indices in harwell-boeing format
+    index_t* hb_col;
 };
 
 
