@@ -1,7 +1,7 @@
 
 /*****************************************************************************
 *
-* Copyright (c) 2003-2020 by The University of Queensland
+* Copyright (c) 2003-2018 by The University of Queensland
 * http://www.uq.edu.au
 *
 * Primary Business: Queensland, Australia
@@ -10,9 +10,8 @@
 *
 * Development until 2012 by Earth Systems Science Computational Center (ESSCC)
 * Development 2012-2013 by School of Earth Sciences
-* Development from 2014-2017 by Centre for Geoscience Computing (GeoComp)
-* Development from 2019 by School of Earth and Environmental Sciences
-**
+* Development from 2014 by Centre for Geoscience Computing (GeoComp)
+*
 *****************************************************************************/
 
 
@@ -44,7 +43,7 @@ void Preconditioner_free(Preconditioner* in)
     }
 }
 
-Preconditioner* Preconditioner_alloc(SystemMatrix_ptr A, Options* options)
+Preconditioner* Preconditioner_alloc(SystemMatrix_ptr<double> A, Options* options)
 {
     Preconditioner* prec = new Preconditioner;
     prec->jacobi=NULL;
@@ -109,7 +108,7 @@ Preconditioner* Preconditioner_alloc(SystemMatrix_ptr A, Options* options)
 /* Applies the preconditioner. */
 /* Has to be called within a parallel region. */
 /* Barrier synchronization is performed before the evaluation to make sure that the input vector is available */
-void Preconditioner_solve(Preconditioner* prec, SystemMatrix_ptr A,
+void Preconditioner_solve(Preconditioner* prec, SystemMatrix_ptr<double> A,
                           double* x, double* b)
 {
     dim_t n=0;
