@@ -1,7 +1,7 @@
 
 /*****************************************************************************
 *
-* Copyright (c) 2003-2020 by The University of Queensland
+* Copyright (c) 2003-2018 by The University of Queensland
 * http://www.uq.edu.au
 *
 * Primary Business: Queensland, Australia
@@ -10,9 +10,8 @@
 *
 * Development until 2012 by Earth Systems Science Computational Center (ESSCC)
 * Development 2012-2013 by School of Earth Sciences
-* Development from 2014-2017 by Centre for Geoscience Computing (GeoComp)
-* Development from 2019 by School of Earth and Environmental Sciences
-**
+* Development from 2014 by Centre for Geoscience Computing (GeoComp)
+*
 *****************************************************************************/
 
 
@@ -69,7 +68,7 @@ to
 
    then RILU is applied to S again until S becomes empty
 */
-Solver_RILU* Solver_getRILU(SparseMatrix_ptr A_p, bool verbose)
+Solver_RILU* Solver_getRILU(SparseMatrix_ptr<double> A_p, bool verbose)
 {
     Solver_RILU* out=NULL;
     dim_t n=A_p->numRows;
@@ -78,7 +77,7 @@ Solver_RILU* Solver_getRILU(SparseMatrix_ptr A_p, bool verbose)
     index_t* counter=NULL;
     index_t iPtr,*index, *where_p;
     dim_t i,k;
-    SparseMatrix_ptr schur;
+    SparseMatrix_ptr<double> schur;
     double A11,A12,A13,A21,A22,A23,A31,A32,A33,D,time0=0,time1=0;/*,time2=0;*/
 
     mis_marker=new index_t[n];
