@@ -1,7 +1,7 @@
 
 /*****************************************************************************
 *
-* Copyright (c) 2003-2020 by The University of Queensland
+* Copyright (c) 2003-2018 by The University of Queensland
 * http://www.uq.edu.au
 *
 * Primary Business: Queensland, Australia
@@ -10,9 +10,8 @@
 *
 * Development until 2012 by Earth Systems Science Computational Center (ESSCC)
 * Development 2012-2013 by School of Earth Sciences
-* Development from 2014-2017 by Centre for Geoscience Computing (GeoComp)
-* Development from 2019 by School of Earth and Environmental Sciences
-**
+* Development from 2014 by Centre for Geoscience Computing (GeoComp)
+*
 *****************************************************************************/
 
 
@@ -26,9 +25,9 @@
 
 namespace paso {
 
-void Solver_updateIncompleteSchurComplement(SparseMatrix_ptr A_CC,
-        SparseMatrix_ptr A_CF, double* invA_FF, index_t* A_FF_pivot,
-        SparseMatrix_ptr A_FC)
+void Solver_updateIncompleteSchurComplement(SparseMatrix_ptr<double> A_CC,
+        SparseMatrix_ptr<double> A_CF, double* invA_FF, index_t* A_FF_pivot,
+        SparseMatrix_ptr<double> A_FC)
 {
   index_t iPtr_CC,*index_CC,col_CF,col_FC, *where_p,iPtr_CC_2,i,iPtr_CF,iPtr_FC;
   dim_t index_CC_len;
