@@ -1,6 +1,6 @@
 /*****************************************************************************
 *
-* Copyright (c) 2003-2020 by The University of Queensland
+* Copyright (c) 2003-2018 by The University of Queensland
 * http://www.uq.edu.au
 *
 * Primary Business: Queensland, Australia
@@ -9,9 +9,8 @@
 *
 * Development until 2012 by Earth Systems Science Computational Center (ESSCC)
 * Development 2012-2013 by School of Earth Sciences
-* Development from 2014-2017 by Centre for Geoscience Computing (GeoComp)
-* Development from 2019 by School of Earth and Environmental Sciences
-**
+* Development from 2014 by Centre for Geoscience Computing (GeoComp)
+*
 *****************************************************************************/
 
 
@@ -38,13 +37,13 @@ namespace bm = boost::math;
 
 namespace paso {
 
-void Solver_free(SystemMatrix* A)
+void Solver_free(SystemMatrix<double>* A)
 {
     A->freePreconditioner();
 }
 
 ///  calls the iterative solver
-SolverResult Solver(SystemMatrix_ptr A, double* x, double* b, Options* options,
+SolverResult Solver(SystemMatrix_ptr<double> A, double* x, double* b, Options* options,
                     Performance* pp)
 {
     const real_t EPSILON = escript::DataTypes::real_t_eps();
@@ -333,5 +332,16 @@ SolverResult Solver(SystemMatrix_ptr A, double* x, double* b, Options* options,
     return errorCode;
 }
 
+SolverResult Solver(SystemMatrix_ptr<cplx_t> A, cplx_t* x, cplx_t* b, Options* options,
+                    Performance* pp)
+{
+    throw PasoException("Solver(): complex not implemented.");
+}
+
+void Solver_free(SystemMatrix<cplx_t>* A)
+{
+    throw PasoException("Solver_free(): complex not implemented.");
+}
+
 } // namespace paso
 
