@@ -1,7 +1,7 @@
 
 /*****************************************************************************
 *
-* Copyright (c) 2003-2020 by The University of Queensland
+* Copyright (c) 2003-2018 by The University of Queensland
 * http://www.uq.edu.au
 *
 * Primary Business: Queensland, Australia
@@ -10,9 +10,8 @@
 *
 * Development until 2012 by Earth Systems Science Computational Center (ESSCC)
 * Development 2012-2013 by School of Earth Sciences
-* Development from 2014-2017 by Centre for Geoscience Computing (GeoComp)
-* Development from 2019 by School of Earth and Environmental Sciences
-**
+* Development from 2014 by Centre for Geoscience Computing (GeoComp)
+*
 *****************************************************************************/
 
 
@@ -41,11 +40,12 @@ namespace paso {
 */
 
 
-SparseMatrix_ptr SparseMatrix::getSubmatrix(dim_t n_row_sub, dim_t n_col_sub,
+template <>
+SparseMatrix_ptr<double> SparseMatrix<double>::getSubmatrix(dim_t n_row_sub, dim_t n_col_sub,
                                             const index_t* row_list,
                                             const index_t* new_col_index) const
 {
-    SparseMatrix_ptr out;
+    SparseMatrix_ptr<double> out;
     if (type & MATRIX_FORMAT_CSC) {
         throw PasoException("SparseMatrix::getSubmatrix: gathering submatrices supports CSR matrix format only.");
     }
@@ -54,7 +54,7 @@ SparseMatrix_ptr SparseMatrix::getSubmatrix(dim_t n_row_sub, dim_t n_col_sub,
     Pattern_ptr sub_pattern(pattern->getSubpattern(n_row_sub, n_col_sub,
                                                    row_list, new_col_index));
     // create the return object
-    out.reset(new SparseMatrix(type, sub_pattern, row_block_size,
+    out.reset(new SparseMatrix<double>(type, sub_pattern, row_block_size,
                                col_block_size, true));
 #pragma omp parallel for
     for (int i=0; i<n_row_sub; ++i) {
@@ -77,11 +77,12 @@ SparseMatrix_ptr SparseMatrix::getSubmatrix(dim_t n_row_sub, dim_t n_col_sub,
     return out;
 }
 
-SparseMatrix_ptr SparseMatrix::getBlock(int blockid) const
+template <>
+SparseMatrix_ptr<double> SparseMatrix<double>::getBlock(int blockid) const
 {
     const dim_t blocksize = row_block_size;
     const dim_t n = numRows;
-    SparseMatrix_ptr out(new SparseMatrix(type, pattern, 1, 1, 0));
+    SparseMatrix_ptr<double> out(new SparseMatrix<double>(type, pattern, 1, 1, 0));
 
     if (blocksize==1) {
         if (blockid==1) {
