@@ -1,7 +1,7 @@
 
 /*****************************************************************************
 *
-* Copyright (c) 2003-2020 by The University of Queensland
+* Copyright (c) 2003-2018 by The University of Queensland
 * http://www.uq.edu.au
 *
 * Primary Business: Queensland, Australia
@@ -10,9 +10,8 @@
 *
 * Development until 2012 by Earth Systems Science Computational Center (ESSCC)
 * Development 2012-2013 by School of Earth Sciences
-* Development from 2014-2017 by Centre for Geoscience Computing (GeoComp)
-* Development from 2019 by School of Earth and Environmental Sciences
-**
+* Development from 2014 by Centre for Geoscience Computing (GeoComp)
+*
 *****************************************************************************/
 
 
@@ -177,7 +176,8 @@ void generate_HB(std::ofstream& fp, dim_t *col_ptr, dim_t *row_ind,
     print_data(fp, val_perline, val_width, nz, val_fmt, val, 0, 0);
 }
 
-void SparseMatrix::saveHB_CSC(const char* filename) const
+template <>
+void SparseMatrix<double>::saveHB_CSC(const char* filename) const
 {
     std::ofstream f(filename);
     if (f.fail()) {
@@ -230,5 +230,11 @@ void SparseMatrix::saveHB_CSC(const char* filename) const
     f.close();
 }
 
+template <>
+void SparseMatrix<cplx_t>::saveHB_CSC(const char* filename) const
+{
+    throw PasoException("SparseMatrix::saveHB_CSC(): complex not implemented.");
+}
+
 } // namespace paso
 
