@@ -1,7 +1,7 @@
 
 /*****************************************************************************
 *
-* Copyright (c) 2003-2020 by The University of Queensland
+* Copyright (c) 2003-2018 by The University of Queensland
 * http://www.uq.edu.au
 *
 * Primary Business: Queensland, Australia
@@ -10,9 +10,8 @@
 *
 * Development until 2012 by Earth Systems Science Computational Center (ESSCC)
 * Development 2012-2013 by School of Earth Sciences
-* Development from 2014-2017 by Centre for Geoscience Computing (GeoComp)
-* Development from 2019 by School of Earth and Environmental Sciences
-**
+* Development from 2014 by Centre for Geoscience Computing (GeoComp)
+*
 *****************************************************************************/
 
 
@@ -31,37 +30,8 @@
 
 namespace paso {
 
-/*  raw scaled vector update operation: out = alpha * A * in + beta * out */
-void SystemMatrix::MatrixVector(double alpha, const double* in, double beta,
-                                double* out) const
-{
-    if (is_balanced) {
-        throw PasoException("MatrixVector: balanced matrix is not supported.");
-    }
-    if (type & MATRIX_FORMAT_CSC) {
-        if (mpi_info->size > 1) {
-            throw PasoException("MatrixVector: CSC is not supported by MPI.");
-        } else {
-            if (type & MATRIX_FORMAT_OFFSET1) {
-                SparseMatrix_MatrixVector_CSC_OFFSET1(alpha, mainBlock, in, beta, out);
-            } else {
-                SparseMatrix_MatrixVector_CSC_OFFSET0(alpha, mainBlock, in, beta, out);
-            }
-        }
-    } else {
-        if (type & MATRIX_FORMAT_OFFSET1) {
-            if (mpi_info->size > 1) {
-                throw PasoException("MatrixVector: CSR with offset 1 is not supported in MPI.");
-            } else {
-                SparseMatrix_MatrixVector_CSR_OFFSET1(alpha, mainBlock, in, beta, out);
-            }
-        } else {
-            MatrixVector_CSR_OFFSET0(alpha, in, beta, out);
-        }
-    }
-}
-
-void SystemMatrix::MatrixVector_CSR_OFFSET0(double alpha, const double* in,
+template <>
+void SystemMatrix<double>::MatrixVector_CSR_OFFSET0(double alpha, const double* in,
                                             double beta, double* out) const
 {
     // start exchange
@@ -84,5 +54,55 @@ void SystemMatrix::MatrixVector_CSR_OFFSET0(double alpha, const double* in,
     }
 }
 
+/*  raw scaled vector update operation: out = alpha * A * in + beta * out */
+template <>
+void SystemMatrix<double>::MatrixVector(double alpha, const double* in, double beta,
+                                double* out) const
+{
+    if (is_balanced) {
+        throw PasoException("MatrixVector: balanced matrix is not supported.");
+    }
+    if (type & MATRIX_FORMAT_CSC) {
+        if (mpi_info->size > 1) {
+            throw PasoException("MatrixVector: CSC is not supported by MPI.");
+        } else {
+            if (type & MATRIX_FORMAT_OFFSET1) {
+                SparseMatrix_MatrixVector_CSC_OFFSET1(alpha, mainBlock, in, beta, out);
+            } else {
+                SparseMatrix_MatrixVector_CSC_OFFSET0(alpha, mainBlock, in, beta, out);
+            }
+        }
+    } else {
+        if (type & MATRIX_FORMAT_OFFSET1) {
+            if (mpi_info->size > 1) {
+                throw PasoException("MatrixVector: CSR with offset 1 is not supported in MPI.");
+            } else {
+                SparseMatrix_MatrixVector_CSR_OFFSET1<double>(alpha, mainBlock, in, beta, out);
+            }
+        } else {
+            MatrixVector_CSR_OFFSET0(alpha, in, beta, out);
+        }
+    }
+}
+
+template <>
+void SystemMatrix<cplx_t>::MatrixVector(double alpha, const cplx_t* in, double beta,
+                                cplx_t* out) const
+{
+#if defined(ESYS_HAVE_MUMPS)
+    if (is_balanced) {
+        throw PasoException("MatrixVector: balanced matrix is not supported.");
+    }
+    if (type & (MATRIX_FORMAT_OFFSET1 + MATRIX_FORMAT_BLK1)) {
+        SparseMatrix_MatrixVector_CSR_OFFSET1<cplx_t>(alpha, mainBlock, in, beta, out);
+        return;
+    } else {
+        throw PasoException("MatrixVector: MUMPS requires CSR format with "
+                            "index offset 1 and block size 1.");
+    }
+#endif
+    throw PasoException("MatrixVector: require MUMPS for complex matrices.");
+}
+
 } // namespace paso
 
