@@ -1,7 +1,7 @@
 
 /*****************************************************************************
 *
-* Copyright (c) 2003-2020 by The University of Queensland
+* Copyright (c) 2003-2018 by The University of Queensland
 * http://www.uq.edu.au
 *
 * Primary Business: Queensland, Australia
@@ -10,9 +10,8 @@
 *
 * Development until 2012 by Earth Systems Science Computational Center (ESSCC)
 * Development 2012-2013 by School of Earth Sciences
-* Development from 2014-2017 by Centre for Geoscience Computing (GeoComp)
-* Development from 2019 by School of Earth and Environmental Sciences
-**
+* Development from 2014 by Centre for Geoscience Computing (GeoComp)
+*
 *****************************************************************************/
 
 
@@ -35,7 +34,8 @@ namespace paso {
 
 // fills the matrix with values i+f1*j where i and j are the global row
 // and column indices of the matrix entry
-void SystemMatrix::fillWithGlobalCoordinates(double f1)
+template <>
+void SystemMatrix<double>::fillWithGlobalCoordinates(double f1)
 {
     const dim_t n = getNumRows();
     const dim_t m = getNumCols();
@@ -98,7 +98,8 @@ void SystemMatrix::fillWithGlobalCoordinates(double f1)
     delete[] rows;
 }
 
-void SystemMatrix::print() const
+template <>
+void SystemMatrix<double>::print() const
 {
     const dim_t n = getNumRows();
     const int rank = mpi_info->rank;
