@@ -1,7 +1,7 @@
 
 /*****************************************************************************
 *
-* Copyright (c) 2003-2020 by The University of Queensland
+* Copyright (c) 2003-2018 by The University of Queensland
 * http://www.uq.edu.au
 *
 * Primary Business: Queensland, Australia
@@ -10,9 +10,8 @@
 *
 * Development until 2012 by Earth Systems Science Computational Center (ESSCC)
 * Development 2012-2013 by School of Earth Sciences
-* Development from 2014-2017 by Centre for Geoscience Computing (GeoComp)
-* Development from 2019 by School of Earth and Environmental Sciences
-**
+* Development from 2014 by Centre for Geoscience Computing (GeoComp)
+*
 *****************************************************************************/
 
 
@@ -98,13 +97,14 @@ void q_sort(index_t *row, index_t *col, double *val, int begin, int end)
     }
 }
 
-SystemMatrix_ptr SystemMatrix::loadMM_toCSR(const char *filename)
+template <>
+SystemMatrix_ptr<double> SystemMatrix<double>::loadMM_toCSR(const char *filename)
 {
     index_t *col_ind = NULL;
     index_t *row_ind = NULL;
     index_t *row_ptr = NULL;
     double *val = NULL;
-    SystemMatrix_ptr out;
+    SystemMatrix_ptr<double> out;
     int curr_row;
     MM_typecode matrixCode;
     escript::JMPI mpi_info = escript::makeInfo(MPI_COMM_WORLD);
@@ -185,7 +185,7 @@ SystemMatrix_ptr SystemMatrix::loadMM_toCSR(const char *filename)
     SystemMatrixPattern_ptr pattern(new SystemMatrixPattern(
                 MATRIX_FORMAT_DEFAULT, output_dist, input_dist, mainPattern,
                 couplePattern, couplePattern, connector, connector));
-    out.reset(new SystemMatrix(MATRIX_FORMAT_DEFAULT, pattern, 1, 1, true,
+    out.reset(new SystemMatrix<double>(MATRIX_FORMAT_DEFAULT, pattern, 1, 1, true,
                 escript::FunctionSpace(), escript::FunctionSpace()));
 
     // copy values
@@ -198,11 +198,12 @@ SystemMatrix_ptr SystemMatrix::loadMM_toCSR(const char *filename)
     return out;
 }
 
-SystemMatrix_ptr SystemMatrix::loadMM_toCSC(const char* filename)
+template <>
+SystemMatrix_ptr<double> SystemMatrix<double>::loadMM_toCSC(const char* filename)
 {
     Pattern_ptr mainPattern, couplePattern;
     SystemMatrixPattern_ptr pattern;
-    SystemMatrix_ptr out;
+    SystemMatrix_ptr<double> out;
     Connector_ptr connector;
     index_t *col_ind = NULL;
     index_t *row_ind = NULL;
@@ -283,7 +284,7 @@ SystemMatrix_ptr SystemMatrix::loadMM_toCSC(const char* filename)
     pattern.reset(new SystemMatrixPattern(MATRIX_FORMAT_DEFAULT,
                 output_dist, input_dist, mainPattern, couplePattern,
                 couplePattern, connector, connector));
-    out.reset(new SystemMatrix(MATRIX_FORMAT_CSC, pattern, 1, 1, true,
+    out.reset(new SystemMatrix<double>(MATRIX_FORMAT_CSC, pattern, 1, 1, true,
                                escript::FunctionSpace(),
                                escript::FunctionSpace()));
 
