@@ -1,7 +1,7 @@
 
 /*****************************************************************************
 *
-* Copyright (c) 2003-2020 by The University of Queensland
+* Copyright (c) 2003-2018 by The University of Queensland
 * http://www.uq.edu.au
 *
 * Primary Business: Queensland, Australia
@@ -10,9 +10,8 @@
 *
 * Development until 2012 by Earth Systems Science Computational Center (ESSCC)
 * Development 2012-2013 by School of Earth Sciences
-* Development from 2014-2017 by Centre for Geoscience Computing (GeoComp)
-* Development from 2019 by School of Earth and Environmental Sciences
-**
+* Development from 2014 by Centre for Geoscience Computing (GeoComp)
+*
 *****************************************************************************/
 
 
@@ -59,10 +58,10 @@ TransportProblem::TransportProblem(SystemMatrixPattern_ptr pattern,
     // at the moment only block size 1 is supported
     SystemMatrixType matrix_type = MATRIX_FORMAT_DEFAULT+MATRIX_FORMAT_BLK1;
 
-    transport_matrix.reset(new SystemMatrix(matrix_type, pattern, block_size,
+    transport_matrix.reset(new SystemMatrix<double>(matrix_type, pattern, block_size,
                                             block_size, false,
                                             functionspace, functionspace));
-    mass_matrix.reset(new SystemMatrix(matrix_type, pattern, block_size,
+    mass_matrix.reset(new SystemMatrix<double>(matrix_type, pattern, block_size,
                                        block_size, false, functionspace,
                                        functionspace));
 
