@@ -1,7 +1,7 @@
 
 /*****************************************************************************
 *
-* Copyright (c) 2003-2020 by The University of Queensland
+* Copyright (c) 2003-2018 by The University of Queensland
 * http://www.uq.edu.au
 *
 * Primary Business: Queensland, Australia
@@ -10,9 +10,8 @@
 *
 * Development until 2012 by Earth Systems Science Computational Center (ESSCC)
 * Development 2012-2013 by School of Earth Sciences
-* Development from 2014-2017 by Centre for Geoscience Computing (GeoComp)
-* Development from 2019 by School of Earth and Environmental Sciences
-**
+* Development from 2014 by Centre for Geoscience Computing (GeoComp)
+*
 *****************************************************************************/
 
 
@@ -33,7 +32,7 @@
 namespace paso {
 
 /// frees any UMFPACK related data from the matrix
-void UMFPACK_free(SparseMatrix* A)
+void UMFPACK_free(SparseMatrix<double>* A)
 {
     if (A && A->solver_p) {
         UMFPACK_Handler* pt = reinterpret_cast<UMFPACK_Handler*>(A->solver_p);
@@ -53,7 +52,7 @@ void UMFPACK_free(SparseMatrix* A)
 
 
 /// calls the solver
-void UMFPACK_solve(SparseMatrix_ptr A, double* out, double* in,
+void UMFPACK_solve(SparseMatrix_ptr<double> A, double* out, double* in,
                    dim_t numRefinements, bool verbose)
 {
 #ifdef ESYS_HAVE_UMFPACK
@@ -203,5 +202,26 @@ void UMFPACK_solve(SparseMatrix_ptr A, double* out, double* in,
 #endif
 }
 
+/// frees any UMFPACK related data from the matrix
+void UMFPACK_free(SparseMatrix<cplx_t>* A)
+{
+#ifdef ESYS_HAVE_UMFPACK
+    throw PasoException("Paso UMFPACK_free(): complex not implemented.");
+#else
+    throw PasoException("Paso: Not compiled with UMFPACK.");
+#endif
+}
+
+/// calls the solver
+void UMFPACK_solve(SparseMatrix_ptr<cplx_t> A, cplx_t* out, cplx_t* in,
+                   dim_t numRefinements, bool verbose)
+{
+#ifdef ESYS_HAVE_UMFPACK
+    throw PasoException("Paso UMFPACK_solve(): complex not implemented.");
+#else
+    throw PasoException("Paso: Not compiled with UMFPACK.");
+#endif
+}
+
 } // namespace paso
 
