@@ -1,7 +1,7 @@
 
 /*****************************************************************************
 *
-* Copyright (c) 2003-2020 by The University of Queensland
+* Copyright (c) 2003-2018 by The University of Queensland
 * http://www.uq.edu.au
 *
 * Primary Business: Queensland, Australia
@@ -10,9 +10,8 @@
 *
 * Development until 2012 by Earth Systems Science Computational Center (ESSCC)
 * Development 2012-2013 by School of Earth Sciences
-* Development from 2014-2017 by Centre for Geoscience Computing (GeoComp)
-* Development from 2019 by School of Earth and Environmental Sciences
-**
+* Development from 2014 by Centre for Geoscience Computing (GeoComp)
+*
 *****************************************************************************/
 
 
@@ -34,10 +33,12 @@
 #include "Solver.h"
 #include "MKL.h"
 #include "UMFPACK.h"
+#include "MUMPS.h"
 
 namespace paso {
 
-void SystemMatrix::solve(double* out, double* in, Options* options) const
+template <>
+void SystemMatrix<double>::solve(double* out, double* in, Options* options) const
 {
     Performance pp;
     index_t package;
@@ -93,6 +94,23 @@ void SystemMatrix::solve(double* out, double* in, Options* options) const
             options->converged = true;
         break;
 
+        case PASO_MUMPS:
+            if (mpi_info->size > 1) {
+                throw PasoException("solve: MUMPS package does not support MPI.");
+            }
+            options->converged = false;
+            options->time = escript::gettime();
+            Performance_startMonitor(&pp, PERFORMANCE_ALL);
+            MUMPS_solve(mainBlock, out, in, options->refinements, options->verbose);
+            solver_package = PASO_MUMPS;
+            Performance_stopMonitor(&pp, PERFORMANCE_ALL);
+            options->time = escript::gettime()-options->time;
+            options->set_up_time = 0;
+            options->residual_norm = 0.;
+            options->num_iter = 0;
+            options->converged = true;
+        break;
+
         default:
             throw PasoException("solve: unknown package code");
         break;
@@ -126,27 +144,69 @@ void SystemMatrix::solve(double* out, double* in, Options* options) const
     Performance_close(&pp, options->verbose);
 }
 
-void solve_free(SystemMatrix* in)
+template <>
+void SystemMatrix<cplx_t>::solve(cplx_t* out, cplx_t* in, Options* options) const
 {
-    if (!in) return;
+    Performance pp;
+    index_t package;
+    if (getGlobalNumCols() != getGlobalNumRows()
+                    || col_block_size != row_block_size) {
+        throw PasoException("solve: matrix has to be a square matrix.");
+    }
+    //options->show();
+    Performance_open(&pp, options->verbose);
+    package = Options::getPackage(options->method, options->package, options->symmetric, mpi_info);
+    SolverResult res = NoError;
 
-    switch(in->solver_package) {
-        case PASO_PASO:
-            Solver_free(in);
-            break;
+    switch (package) {
+        case PASO_MUMPS:
+            if (mpi_info->size > 1) {
+                throw PasoException("solve: MUMPS package does not support MPI.");
+            }
+            options->converged = false;
+            options->time = escript::gettime();
+            Performance_startMonitor(&pp, PERFORMANCE_ALL);
+            MUMPS_solve(mainBlock, out, in, options->refinements, options->verbose);
+            solver_package = PASO_MUMPS;
+            Performance_stopMonitor(&pp, PERFORMANCE_ALL);
+            options->time = escript::gettime()-options->time;
+            options->set_up_time = 0;
+            options->residual_norm = 0.;
+            options->num_iter = 0;
+            options->converged = true;
+        break;
 
-        case PASO_SMOOTHER:
-            Preconditioner_Smoother_free((Preconditioner_Smoother*) in->solver_p);
-            break;
+        default:
+            throw PasoException("solve: MUMPS required for complex matrices.");
+        break;
+    }
 
-        case PASO_MKL:
-            MKL_free(in->mainBlock.get());
-            break;
-
-        case PASO_UMFPACK:
-            UMFPACK_free(in->mainBlock.get());
-            break;
-   }
+    if (res == Divergence) {
+        // cancel divergence errors
+        if (options->accept_failed_convergence) {
+            if (options->verbose)
+                printf("paso: failed convergence error has been canceled as requested.\n");
+        } else {
+            throw PasoException("Solver: No improvement during iteration. Iterative solver gives up.");
+        }
+    } else if (res == MaxIterReached) {
+        // cancel divergence errors
+        if (options->accept_failed_convergence) {
+            if (options->verbose)
+                printf("paso: failed convergence error has been canceled as requested.\n");
+        } else {
+            throw PasoException("Solver: maximum number of iteration steps reached.\nReturned solution does not fulfil stopping criterion.");
+        }
+    } else if (res == InputError) {
+        throw PasoException("Solver: illegal dimension in iterative solver.");
+    } else if (res == NegativeNormError) {
+        throw PasoException("Solver: negative energy norm (try other solver or preconditioner).");
+    } else if (res == Breakdown) {
+        throw PasoException("Solver: fatal break down in iterative solver.");
+    } else if (res != NoError) {
+        throw PasoException("Solver: Generic error in solver.");
+    }
+    Performance_close(&pp, options->verbose);
 }
 
 } // namespace paso
