@@ -917,13 +917,20 @@ int RipleyDomain::getSystemMatrixTypeId(const bp::object& options) const
 #endif
     }
 #ifdef ESYS_HAVE_PASO
-    if (sb.isComplex()) {
-        throw NotImplementedError("Paso does not support complex-valued matrices");
-    }
     // in all other cases we use PASO
-    return (int)SMT_PASO | paso::SystemMatrix::getSystemMatrixTypeId(
-            method, sb.getPreconditioner(), sb.getPackage(),
-            sb.isSymmetric(), m_mpiInfo);
+    if (sb.isComplex()) {
+#ifdef ESYS_HAVE_MUMPS
+        return (int)SMT_PASO | paso::SystemMatrix<cplx_t>::getSystemMatrixTypeId(
+                method, sb.getPreconditioner(), sb.getPackage(),
+                sb.isComplex(), sb.isSymmetric(), m_mpiInfo);
+#else
+        throw NotImplementedError("Paso requires MUMPS for complex-valued matrices.");
+#endif
+    } else {
+        return (int)SMT_PASO | paso::SystemMatrix<double>::getSystemMatrixTypeId(
+                method, sb.getPreconditioner(), sb.getPackage(),
+                sb.isComplex(), sb.isSymmetric(), m_mpiInfo);
+    }
 #else
     throw RipleyException("Unable to find a working solver library!");
 #endif
@@ -994,10 +1001,17 @@ escript::ASM_ptr RipleyDomain::newSystemMatrix(int row_blocksize,
         paso::SystemMatrixPattern_ptr pattern(getPasoMatrixPattern(
                                             reduceRowOrder, reduceColOrder));
         type -= (int)SMT_PASO;
-        escript::ASM_ptr sm(new paso::SystemMatrix(type, pattern,
-                row_blocksize, column_blocksize, false, row_functionspace,
-                column_functionspace));
-        return sm;
+        if (type & (int)MATRIX_FORMAT_COMPLEX) {
+            escript::ASM_ptr sm(new paso::SystemMatrix<cplx_t>(type, pattern,
+                    row_blocksize, column_blocksize, false, row_functionspace,
+                    column_functionspace));
+            return sm;
+        } else {
+            escript::ASM_ptr sm(new paso::SystemMatrix<double>(type, pattern,
+                    row_blocksize, column_blocksize, false, row_functionspace,
+                    column_functionspace));
+            return sm;
+        }
 #else
         throw RipleyException("newSystemMatrix: ripley was not compiled with "
                "Paso support so the Paso solver stack cannot be used.");
@@ -1421,7 +1435,7 @@ void RipleyDomain::addToSystemMatrix<real_t>(escript::AbstractSystemMatrix* mat,
                                          const DoubleVector& array) const
 {
 #ifdef ESYS_HAVE_PASO
-    paso::SystemMatrix* psm = dynamic_cast<paso::SystemMatrix*>(mat);
+    paso::SystemMatrix<real_t>* psm = dynamic_cast<paso::SystemMatrix<real_t>*>(mat);
     if (psm) {
         addToPasoMatrix(psm, nodes, numEq, array);
         return;
@@ -1450,6 +1464,13 @@ void RipleyDomain::addToSystemMatrix<cplx_t>(escript::AbstractSystemMatrix* mat,
                                          const IndexVector& nodes, dim_t numEq,
                                          const vector<cplx_t>& array) const
 {
+#ifdef ESYS_HAVE_MUMPS
+    paso::SystemMatrix<cplx_t>* psm = dynamic_cast<paso::SystemMatrix<cplx_t>*>(mat);
+    if (psm) {
+        addToPasoMatrix(psm, nodes, numEq, array);
+        return;
+    }
+#endif
 #ifdef ESYS_HAVE_TRILINOS
     TrilinosMatrixAdapter* tm = dynamic_cast<TrilinosMatrixAdapter*>(mat);
     if (tm) {
@@ -1457,15 +1478,16 @@ void RipleyDomain::addToSystemMatrix<cplx_t>(escript::AbstractSystemMatrix* mat,
         return;
     }
 #endif
-    throw RipleyException("addToSystemMatrix: only Trilinos matrices support "
+    throw RipleyException("addToSystemMatrix: require Trilinos or MUMPS matrices for "
                           "complex-valued assembly!");
 }
 
 #ifdef ESYS_HAVE_PASO
 //private
-void RipleyDomain::addToPasoMatrix(paso::SystemMatrix* mat,
+template <typename T>
+void RipleyDomain::addToPasoMatrix(paso::SystemMatrix<T>* mat,
                                    const IndexVector& nodes, dim_t numEq,
-                                   const vector<double>& array) const
+                                   const vector<T>& array) const
 {
     const dim_t numMyCols = mat->pattern->mainPattern->numInput;
     const dim_t numMyRows = mat->pattern->mainPattern->numOutput;
@@ -1474,13 +1496,13 @@ void RipleyDomain::addToPasoMatrix(paso::SystemMatrix* mat,
 
     const index_t* mainBlock_ptr = mat->mainBlock->pattern->ptr;
     const index_t* mainBlock_index = mat->mainBlock->pattern->index;
-    double* mainBlock_val = mat->mainBlock->val;
+    T* mainBlock_val = mat->mainBlock->val;
     const index_t* col_coupleBlock_ptr = mat->col_coupleBlock->pattern->ptr;
     const index_t* col_coupleBlock_index = mat->col_coupleBlock->pattern->index;
-    double* col_coupleBlock_val = mat->col_coupleBlock->val;
+    T* col_coupleBlock_val = mat->col_coupleBlock->val;
     const index_t* row_coupleBlock_ptr = mat->row_coupleBlock->pattern->ptr;
     const index_t* row_coupleBlock_index = mat->row_coupleBlock->pattern->index;
-    double* row_coupleBlock_val = mat->row_coupleBlock->val;
+    T* row_coupleBlock_val = mat->row_coupleBlock->val;
     index_t offset=(mat->type & MATRIX_FORMAT_OFFSET1 ? 1:0);
 
 #define UPDATE_BLOCK(VAL) do {\
@@ -1825,3 +1847,4 @@ void RipleyDomain::addPoints(const vector<double>& coords,
 }
 
 } // end of namespace ripley
+
