@@ -22,10 +22,14 @@ escript_opts_version = 203
 openmp = True
 netcdf = True
 compressed_files = False
-boost_libs = ['boost_python']
+boost_libs = ['boost_python3']
+boost_prefix=['/usr/include/','/usr/lib64']
+netcdf='no'
+pythoncmd='/usr/bin/python3'
 pythonlibpath = ['/usr/lib64']
-pythonlibname = ['python2.7']
-pythonincpath = ['/usr/include/python2.7']
+pythonlibname = ['python3.6m']
+pythonincpath = ['/usr/include/python3.6m']
 cxx_extra = '-std=c++11'
 umfpack = True
-umfpack_prefix = ['/usr/include/suitesparse','/usr/lib']
\ No newline at end of file
+umfpack_prefix = ['/usr/include/suitesparse','/usr/lib']
+
