@@ -99,9 +99,11 @@ def get_external_python_sympy(env,bin):
     if int(tmp1) == 0 and int(tmp2) < 7:
         env['sympy'] = False
         env['warnings'].append("sympy version is too old.")
+        env.Append(CPPDEFINES = ['ESYS_NO_SYMPY'])
     if (int(tmp1) == 1 and int(tmp2) >= 2) or int(tmp1) >= 2:
         env['sympy']=False
         env['warnings'].append("escript does not support sympy version 1.2 and higher. Found version %d.%d" % (int(tmp1),int(tmp2)))
+        env.Append(CPPDEFINES = ['ESYS_NO_SYMPY'])
 
     return env
 
@@ -180,8 +182,24 @@ def checkPython(env):
     # if we want to use a python other than the one scons is running
     # Note: we assume scons is running python 2 in the following.
     else:
-        (python_lib_path, python_libs,verstring, python_inc_path)=call_python_config(env['pythoncmd'])
-
+        if env['IS_WINDOWS']:
+            cmd = "import os, sys\n"
+            cmd += "from distutils import sysconfig\n"
+            cmd += "print(sysconfig.get_python_inc())\n"
+            cmd += "print(os.path.join(sysconfig.get_config_var('prefix'), 'libs'))\n"
+            cmd += "print('python%s%s'%(sys.version_info[0], sys.version_info[1]))\n"
+            cmd += "print('.'.join([str(i) for i in sys.version_info[:3]]))"
+            pout = subprocess.Popen([env['pythoncmd'], '-c', cmd], stdout=subprocess.PIPE).stdout.read()
+            if isinstance(pout, bytes):
+                pout = pout.decode(sys.stdout.encoding)
+            lines = pout.split('\n')
+            python_inc_path = lines[0].strip()
+            python_lib_path = lines[1].strip()
+            python_libs = [lines[2].strip()]
+            verstring = lines[3].strip()
+        else:
+            (python_lib_path, python_libs,verstring, python_inc_path)=call_python_config(env['pythoncmd'])
+    
     if sys.version_info[0] == 3:
         if isinstance(verstring, str) is False:
             verstring = str(verstring, 'utf-8')
@@ -287,47 +305,55 @@ def checkBoost(env):
             boost_numpy_inc_path,boost_numpy_lib_path=findLibWithHeader(env, env['boost_libs'], 'boost/python/numpy.hpp', env['boost_prefix'], lang='c++')
 
             # Locate the boost numpy files
-            p = subprocess.Popen(["ld","--verbose"], stdout=subprocess.PIPE)
-            out,err = p.communicate()
-            spath = [x[13:-3] for x in out.split() if b'SEARCH_DIR' in x]
-            spath.append(boost_lib_path)
-            p2name = ''
-            p3name = ''
-            for name in spath:
-                try:
-                    l=os.listdir(name)
-
-                    import sys
-                    if sys.version_info[0] == 3:
-                        string_type = str
-                    else:
-                        string_type = basestring
-
-                    p2res = ''
-                    p3res = ''
-                    for x in l:
-                        if isinstance(x,string_type):
-                            if (x.startswith('libboost_numpy2')) and x.endswith('.so'):
-                                p2res = x
-                            if (x.startswith('libboost_numpy3')) and x.endswith('.so'):
-                                p3res = x
-                        else:
-                            if (x.startswith(b'libboost_numpy2')) and x.endswith(b'.so'):
-                                p2res = x
-                            if (x.startswith(b'libboost_numpy3')) and x.endswith(b'.so'):
-                                p3res = x
-                    if len(p2name)==0 and len(p2res)>0:
-                        p2name=p2res[3:-3]
-                    if len(p3name)==0 and len(p3res)>0:
-                        p3name=p3res[3:-3]
-                except OSError:
-                    pass
-
-            # Pick the right one
-            if int(env['python_version'][0]) == 2:
-                libname = p2name
+            if env['IS_WINDOWS']:
+                raise Exception # TODO: fix boost numpy dll ex/import compile errors
+                import sys
+                for l in os.listdir(env['boost_prefix'] + '\\lib'):
+                    if l.startswith('boost_numpy{}{}'.format(sys.version_info.major,sys.version_info.minor)):
+                        libname = os.path.splitext(l)[0]
             else:
-                libname = p3name
+                p = subprocess.Popen(["ld","--verbose"], stdout=subprocess.PIPE)
+                out,err = p.communicate()
+                spath = [x[13:-3] for x in out.split() if b'SEARCH_DIR' in x]
+                spath.append(boost_lib_path)
+                p2name = ''
+                p3name = ''
+                for name in spath:
+                    try:
+                        l=os.listdir(name)
+
+                        import sys
+                        if sys.version_info[0] == 3:
+                            string_type = str
+                        else:
+                            string_type = basestring
+
+                        p2res = ''
+                        p3res = ''
+                        for x in l:
+                            if isinstance(x,string_type):
+                                if x.startswith('libboost_numpy-py') and x.endswith('.so'):
+                                    p2res = x
+                                if x.startswith('libboost_numpy-py3') and x.endswith('.so'):
+                                    p3res = x
+                            else:
+                                if x.startswith(b'libboost_numpy-py') and x.endswith(b'.so'):
+                                    p2res = x
+                                if x.startswith(b'libboost_numpy-py3') and x.endswith(b'.so'):
+                                    p3res = x
+
+                        if len(p2name)==0 and len(p2res)>0:
+                            p2name=p2res[-1]
+                        if len(p3name)==0 and len(p3res)>0:
+                            p3name=p3res[-1]
+                    except OSError:
+                        pass
+
+                # Pick the right one
+                if int(env['python_version'][0]) == 2:
+                    libname = p2name[3:-3]
+                else:
+                    libname = p3name[3:-3]
 
             # If found, add the necessary information to env
             if len(libname) > 0:
@@ -347,6 +373,8 @@ def checkBoost(env):
     # Check if the version of boost we are using is missing BOOST_BYTE_ORDER
     if boostversion >= 107000:
         env.Append(CPPDEFINES=['ESYS_DEPRECATED_BOOST_ENDIAN'])
+    if boostversion >= 107200:
+        env.Append(CPPDEFINES=['ESYS_MOVED_BOOST_ENDIAN'])
 
     return env
 
@@ -404,6 +432,7 @@ def checkOptionalModules(env):
     ######## sympy
     if not detectModule(env, 'sympy'):
         env['warnings'].append("Cannot import sympy. Symbolic toolbox and nonlinear PDEs will not be available.")
+        env.Append(CPPDEFINES = ['ESYS_NO_SYMPY'])
     else:
         if env['pythoncmd'] is not None:
             env=get_external_python_sympy(env, env['pythoncmd'])
@@ -415,9 +444,11 @@ def checkOptionalModules(env):
             if duv.LooseVersion(sympy.__version__) < duv.LooseVersion('0.7'):
                 env['sympy']=False
                 env['warnings'].append("sympy version too old. Symbolic toolbox and nonlinear PDEs will not be available.")
+                env.Append(CPPDEFINES = ['ESYS_NO_SYMPY'])
             if duv.LooseVersion(sympy.__version__) > duv.LooseVersion('1.2'):
                 env['sympy']=False
                 env['warnings'].append("escript does not support sympy version 1.2 and higher. Found %d" % duv.LooseVersion(sympy.__version__))
+                env.Append(CPPDEFINES = ['ESYS_NO_SYMPY'])
 
     ######## gmshpy
     env['gmshpy'] = detectModule(env, 'gmshpy')
@@ -458,27 +489,27 @@ def checkForTrilinos(env):
             print("Checking for %s... %s" % ('Tpetra_DefaultPlatform.hpp', "yes" if os.path.isfile(os.path.join(trilinos_inc_path,'Tpetra_DefaultPlatform.hpp')) else "no"))
             env.Append(CPPDEFINES = ['ESYS_HAVE_TPETRA_DP'])
 
-        if os.path.isfile(os.path.join(trilinos_inc_path,'Tpetra_Experimental_BlockCrsMatrix.hpp')):
+        if os.path.isfile(os.path.join(trilinos_inc_path,'Tpetra_BlockCrsMatrix.hpp')):
+            print("Checking for %s... %s" % ('Tpetra_BlockCrsMatrix.hpp', "yes" if os.path.isfile(os.path.join(trilinos_inc_path,'Tpetra_DefaultPlatform.hpp')) else "no"))
+        elif os.path.isfile(os.path.join(trilinos_inc_path,'Tpetra_Experimental_BlockCrsMatrix.hpp')):
             print("Checking for %s... %s" % ('Tpetra_Experimental_BlockCrsMatrix.hpp', "yes" if os.path.isfile(os.path.join(trilinos_inc_path,'Tpetra_DefaultPlatform.hpp')) else "no"))
             env.Append(CPPDEFINES = ['ESYS_HAVE_TPETRA_EXPERIMENTAL_BLOCKCRS'])
-        elif os.path.isfile(os.path.join(trilinos_inc_path,'Tpetra_BlockCrsMatrix.hpp')):
-            print("Checking for %s... %s" % ('Tpetra_BlockCrsMatrix.hpp', "yes" if os.path.isfile(os.path.join(trilinos_inc_path,'Tpetra_DefaultPlatform.hpp')) else "no"))
         else:
             raise RuntimeError('Could not locate the Trilinos Block CRS Matrix header')
 
-        if os.path.isfile(os.path.join(trilinos_inc_path,'Tpetra_Experimental_BlockCrsMatrix_Helpers.hpp')):
+        if os.path.isfile(os.path.join(trilinos_inc_path,'Tpetra_BlockCrsMatrix_Helpers.hpp')):
+            print("Checking for %s... %s" % ('Tpetra_BlockCrsMatrix_Helpers.hpp', "yes" if os.path.isfile(os.path.join(trilinos_inc_path,'Tpetra_DefaultPlatform.hpp')) else "no"))
+        elif os.path.isfile(os.path.join(trilinos_inc_path,'Tpetra_Experimental_BlockCrsMatrix_Helpers.hpp')):
             print("Checking for %s... %s" % ('Tpetra_Experimental_BlockCrsMatrix_Helpers.hpp', "yes" if os.path.isfile(os.path.join(trilinos_inc_path,'Tpetra_DefaultPlatform.hpp')) else "no"))
             env.Append(CPPDEFINES = ['ESYS_HAVE_TPETRA_EXPERIMENTAL_BLOCKCRSH'])
-        elif os.path.isfile(os.path.join(trilinos_inc_path,'Tpetra_BlockCrsMatrix_Helpers.hpp')):
-            print("Checking for %s... %s" % ('Tpetra_BlockCrsMatrix_Helpers.hpp', "yes" if os.path.isfile(os.path.join(trilinos_inc_path,'Tpetra_DefaultPlatform.hpp')) else "no"))
         else:
             raise RuntimeError('Could not locate the Trilinos Block CRS Matrix Helpers header')
 
-        if os.path.isfile(os.path.join(trilinos_inc_path,'Tpetra_Experimental_BlockVector.hpp')):
+        if os.path.isfile(os.path.join(trilinos_inc_path,'Tpetra_BlockVector.hpp')):
+            print("Checking for %s... %s" % ('Tpetra_BlockVector.hpp', "yes" if os.path.isfile(os.path.join(trilinos_inc_path,'Tpetra_DefaultPlatform.hpp')) else "no"))
+        elif os.path.isfile(os.path.join(trilinos_inc_path,'Tpetra_Experimental_BlockVector.hpp')):
             print("Checking for %s... %s" % ('Tpetra_Experimental_BlockVector.hpp', "yes" if os.path.isfile(os.path.join(trilinos_inc_path,'Tpetra_DefaultPlatform.hpp')) else "no"))
             env.Append(CPPDEFINES = ['ESYS_HAVE_TPETRA_EXPERIMENTAL_BLOCKV'])
-        elif os.path.isfile(os.path.join(trilinos_inc_path,'Tpetra_BlockVector.hpp')):
-            print("Checking for %s... %s" % ('Tpetra_BlockVector.hpp', "yes" if os.path.isfile(os.path.join(trilinos_inc_path,'Tpetra_DefaultPlatform.hpp')) else "no"))
         else:
             raise RuntimeError('Could not locate the Trilinos BlockVector header')
 
@@ -568,6 +599,19 @@ def checkOptionalLibraries(env):
         env['buildvars']['umfpack_lib_path']=umfpack_lib_path
     env['buildvars']['umfpack']=int(env['umfpack'])
 
+    ######## MUMPS
+    mumps_inc_path=''
+    mumps_lib_path=''
+    if env['mumps']:
+        mumps_inc_path,mumps_lib_path=findLibWithHeader(env, env['mumps_libs'], 'mumps_mpi.h', env['mumps_prefix'], lang='c++')
+        env.AppendUnique(CPPPATH = [mumps_inc_path])
+        env.AppendUnique(LIBPATH = [mumps_lib_path])
+        env.PrependENVPath(env['LD_LIBRARY_PATH_KEY'], mumps_lib_path)
+        env.Append(CPPDEFINES = ['ESYS_HAVE_MUMPS'])
+        env['buildvars']['mumps_inc_path']=mumps_inc_path
+        env['buildvars']['mumps_lib_path']=mumps_lib_path
+    env['buildvars']['mumps']=int(env['mumps'])
+
     ######## LAPACK
     lapack_inc_path=''
     lapack_lib_path=''
@@ -669,8 +713,10 @@ def checkOptionalLibraries(env):
                 p = Popen(['orterun', '-V'], stdout=PIPE, stderr=PIPE)
                 o,e = p.communicate()
                 try:
+                    e=e.decode()
                     ver = e.split('\n')[0].split()[-1]
                 except IndexError:
+                    o=o.decode()
                     ver = o.split('\n')[0].split()[-1]
                 if len(ver) > 0:
                     env['orte_version'] = ver
@@ -736,8 +782,10 @@ def checkOptionalLibraries(env):
             try:
                 p=Popen(['gmsh', '-info'], stderr=PIPE)
                 _,e=p.communicate()
+                env.Append(CPPDEFINES=['ESYS_HAVE_GMSH'])
                 if e.split().count("MPI"):
                     env['gmsh']='m'
+                    env.Append(CPPDEFINES=['ESYS_GMSH_MPI'])
                 else:
                     env['gmsh']='s'
             except OSError:
@@ -752,8 +800,9 @@ def checkOptionalLibraries(env):
                 try:
                     p=Popen(cmd, stdout=PIPE)
                     gmshlibs,_ = p.communicate()
+                    gmshlibs.decode()
                     env.Append(CPPDEFINES=['ESYS_HAVE_GMSH'])
-                    if p.returncode == 0 and 'libmpi' in gmshlibs:
+                    if p.returncode == 0 and 'libmpi' in gmshlibs.decode('utf-8'):
                         env['gmsh'] = 'm'
                         env.Append(CPPDEFINES=['ESYS_GMSH_MPI'])
                     else:
